"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_types_1 = require("@salesforce/ts-types");
const command_1 = require("@salesforce/command");
const rimraf = __importStar(require("rimraf"));
const jsforce_1 = require("jsforce");
const core_1 = require("@salesforce/core");
// tslint:disable-next-line:ordered-imports
var jsforce = require("jsforce");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "auth_login");
let Login = /** @class */ (() => {
    class Login extends command_1.SfdxCommand {
        async run() {
            rimraf.sync("temp_sfpowerkit");
            if (this.flags.url)
                this.loginUrl = this.flags.url;
            else
                this.loginUrl = "https://test.salesforce.com";
            if (this.flags.securitytoken)
                this.password = this.flags.password.concat(this.flags.securitytoken);
            else
                this.password = this.flags.password;
            let conn = new jsforce_1.Connection({
                loginUrl: this.loginUrl
            });
            await conn.login(this.flags.username, this.password, function (err, userInfo) {
                if (err) {
                    throw new core_1.SfdxError("Unable to connect to the target org");
                }
            });
            const accessTokenOptions = {
                accessToken: conn.accessToken,
                instanceUrl: conn.instanceUrl,
                loginUrl: this.loginUrl,
                orgId: ts_types_1.getString(conn, "userInfo.organizationId")
            };
            const auth = await core_1.AuthInfo.create({
                username: this.flags.username,
                accessTokenOptions
            });
            await auth.save();
            if (this.flags.alias) {
                const aliases = await core_1.Aliases.create(core_1.ConfigGroup.getOptions("orgs", "alias.json"));
                aliases.set(this.flags.alias, this.flags.username);
                await aliases.write();
            }
            this.ux.log(`Authorized to ${this.flags.username}`);
            return { username: this.flags.username, accessTokenOptions };
        }
    }
    Login.description = messages.getMessage("commandDescription");
    Login.examples = [
        `$ sfdx sfpowerkit:auth:login -u azlam@sfdc.com -p Xasdax2w2  -a prod
      Authorized to azlam@sfdc.com
  `
    ];
    Login.flagsConfig = {
        username: command_1.flags.string({
            required: true,
            char: "u",
            description: messages.getMessage("usernameFlagDescription")
        }),
        password: command_1.flags.string({
            required: true,
            char: "p",
            description: messages.getMessage("passwordFlagDescription")
        }),
        securitytoken: command_1.flags.string({
            required: false,
            char: "s",
            description: messages.getMessage("securityTokenFlagDescription")
        }),
        url: command_1.flags.url({
            required: false,
            char: "r",
            description: messages.getMessage("urlFlagDescription")
        }),
        alias: command_1.flags.string({
            required: false,
            char: "a",
            description: messages.getMessage("aliasFlagDescription")
        })
    };
    return Login;
})();
exports.default = Login;
//# sourceMappingURL=login.js.map