"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const dependencyImpl_1 = __importDefault(require("../../../../impl/dependency/dependencyImpl"));
const metadataSummaryInfoFetcher_1 = __importDefault(require("../../../../impl/metadata/retriever/metadataSummaryInfoFetcher"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const fs = __importStar(require("fs-extra"));
const fileutils_1 = __importDefault(require("../../../../utils/fileutils"));
const rimraf = __importStar(require("rimraf"));
const packageInfo_1 = __importDefault(require("../../../../impl/package/version/packageInfo"));
const getDefaults_1 = __importDefault(require("../../../../utils/getDefaults"));
const progressBar_1 = require("../../../../ui/progressBar");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "dependency_tree_package");
let Tree = /** @class */ (() => {
    class Tree extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            this.conn = this.org.getConnection();
            this.output = [];
            //Fetch Package Details from an Org
            try {
                let packageDetails = await new packageInfo_1.default(this.conn, getDefaults_1.default.getApiVersion(), false).getPackages();
                this.installedPackagesMap = new Map(packageDetails.map(obj => [obj.subcriberPackageId, obj]));
            }
            catch (error) {
                throw new core_1.SfdxError("Unable to retrieve details about packages in the org");
            }
            //Find Requested Package
            let requestPackage;
            for (let pkg of this.installedPackagesMap.values()) {
                if (pkg.subcriberPackageId === this.flags.package ||
                    pkg.packageName === this.flags.package ||
                    pkg.packageVersionId === this.flags.package) {
                    requestPackage = pkg;
                    break;
                }
            }
            sfpowerkit_1.SFPowerkit.log("Requested Package Info:" + JSON.stringify(requestPackage), sfpowerkit_1.LoggerLevel.TRACE);
            if (!requestPackage) {
                throw new core_1.SfdxError(`Unable to find the package ${this.flags.package} in ${this.org.getUsername()} org.`);
            }
            sfpowerkit_1.SFPowerkit.log(`Fetching all components details of ${requestPackage.packageName} package from the org`, sfpowerkit_1.LoggerLevel.INFO);
            let packageMembers = await dependencyImpl_1.default.getMemberFromPackage(this.conn, requestPackage.subcriberPackageId);
            sfpowerkit_1.SFPowerkit.log("Package Member Info:" + JSON.stringify(packageMembers), sfpowerkit_1.LoggerLevel.TRACE);
            sfpowerkit_1.SFPowerkit.log(`Found ${packageMembers.length} components from ${requestPackage.packageName} package`, sfpowerkit_1.LoggerLevel.INFO);
            let dependencyResult = await dependencyImpl_1.default.getDependencyMapById(this.conn, packageMembers);
            this.dependencyMap = dependencyResult.dependencyMap;
            this.metadataMap = dependencyResult.dependencyDetailsMap;
            sfpowerkit_1.SFPowerkit.log(`Found ${this.dependencyMap.size} components having dependency`, sfpowerkit_1.LoggerLevel.INFO);
            let result = await metadataSummaryInfoFetcher_1.default.fetchMetadataSummaryFromAnOrg(this.conn);
            for (let metaObj of result.keys())
                this.metadataMap.set(metaObj, result.get(metaObj));
            let membersWithoutDependency = packageMembers.filter(x => !Array.from(this.dependencyMap.keys()).includes(x));
            await this.getDetailsFromId(this.flags.packagefilter, membersWithoutDependency);
            if (this.flags.format === "json") {
                await this.generateJsonOutput(this.output, this.flags.output);
            }
            else {
                await this.generateCSVOutput(this.output, this.flags.output);
            }
            return this.output;
        }
        async getDetailsFromId(packagefilter, membersWithoutDependency) {
            let pkgMemberMap = await dependencyImpl_1.default.getMemberVsPackageMap(this.conn);
            let result = [];
            let progressBar = new progressBar_1.ProgressBar().create(`Computing the dependency tree`, ` items`, sfpowerkit_1.LoggerLevel.INFO);
            progressBar.start(this.flags.showall
                ? this.dependencyMap.size + membersWithoutDependency.length
                : this.dependencyMap.size);
            //items having dependency
            for (let member of this.dependencyMap.keys()) {
                let currentItem = JSON.parse(JSON.stringify(this.metadataMap.has(member)
                    ? this.metadataMap.get(member)
                    : { id: member, fullName: "unknown", type: "unknown" }));
                if (!packagefilter) {
                    currentItem.dependentMetadata = [];
                }
                else {
                    currentItem.dependentPackage = [];
                }
                for (let dependent of this.dependencyMap.get(member)) {
                    let dependentItem = JSON.parse(JSON.stringify(this.metadataMap.has(dependent)
                        ? this.metadataMap.get(dependent)
                        : { id: dependent, fullName: "unknown", type: "unknown" }));
                    dependentItem.package = pkgMemberMap.has(dependent)
                        ? this.installedPackagesMap.get(pkgMemberMap.get(dependent))
                            .packageName
                        : "Org";
                    if (packagefilter &&
                        !currentItem.dependentPackage.includes(dependentItem.package)) {
                        currentItem.dependentPackage.push(dependentItem.package);
                    }
                    if (!packagefilter) {
                        currentItem.dependentMetadata.push(dependentItem);
                    }
                }
                progressBar.increment(1);
                result.push(currentItem);
            }
            if (this.flags.showall) {
                //items with dependency
                membersWithoutDependency.forEach(member => {
                    let currentItem = JSON.parse(JSON.stringify(this.metadataMap.has(member)
                        ? this.metadataMap.get(member)
                        : { id: member, fullName: "unknown", type: "unknown" }));
                    if (!packagefilter) {
                        currentItem.dependentMetadata = [];
                    }
                    else {
                        currentItem.dependentPackage = [];
                    }
                    progressBar.increment(1);
                    result.push(currentItem);
                });
            }
            progressBar.stop();
            this.output = result;
        }
        async generateJsonOutput(result, outputDir) {
            let outputJsonPath = `${outputDir}/output.json`;
            rimraf.sync(outputJsonPath);
            let dir = path.parse(outputJsonPath).dir;
            if (!fs.existsSync(dir)) {
                fileutils_1.default.mkDirByPathSync(dir);
            }
            fs.writeFileSync(outputJsonPath, JSON.stringify(result));
            sfpowerkit_1.SFPowerkit.log(`Output ${outputDir}/output.json is generated successfully`, sfpowerkit_1.LoggerLevel.INFO);
        }
        async generateCSVOutput(result, outputDir) {
            let outputcsvPath = `${outputDir}/output.csv`;
            let dir = path.parse(outputcsvPath).dir;
            if (!fs.existsSync(dir)) {
                fileutils_1.default.mkDirByPathSync(dir);
            }
            let newLine = "\r\n";
            let output = "ID,NAME,TYPE," +
                (this.flags.packagefilter
                    ? "DEPENDENT PACKAGE"
                    : "DEPENDENT ID,DEPENDENT NAME,DEPENDENT TYPE,DEPENDENT PACKAGE") +
                newLine;
            result.forEach(element => {
                if (element.dependentMetadata && element.dependentMetadata.length > 0) {
                    for (let dependent of element.dependentMetadata) {
                        output = `${output}${element.id},${element.fullName},${element.type},${dependent.id},${dependent.fullName},${dependent.type},${dependent.package}${newLine}`;
                    }
                }
                else if (element.dependentPackage &&
                    element.dependentPackage.length > 0) {
                    for (let dependent of element.dependentPackage) {
                        output = `${output}${element.id},${element.fullName},${element.type},${dependent}${newLine}`;
                    }
                }
                else {
                    output = `${output}${element.id},${element.fullName},${element.type}${newLine}`;
                }
            });
            fs.writeFileSync(outputcsvPath, output);
            sfpowerkit_1.SFPowerkit.log(`Output ${outputDir}/output.csv is generated successfully`, sfpowerkit_1.LoggerLevel.INFO);
        }
    }
    Tree.description = messages.getMessage("commandDescription");
    Tree.examples = [
        "$ sfdx sfpowerkit:dependency:tree:package -u MyScratchOrg -n 04txxxxxxxxxx -o outputdir -f json",
        "$ sfdx sfpowerkit:dependency:tree:package -u MyScratchOrg -n 04txxxxxxxxxx -o outputdir -f csv",
        "$ sfdx sfpowerkit:dependency:tree:package -u MyScratchOrg -n 04txxxxxxxxxx -o outputdir -f csv -p",
        "$ sfdx sfpowerkit:dependency:tree:package -u MyScratchOrg -n 04txxxxxxxxxx -o outputdir -f csv -s"
    ];
    Tree.flagsConfig = {
        package: command_1.flags.string({
            char: "n",
            required: true,
            description: messages.getMessage("packageDescription")
        }),
        packagefilter: command_1.flags.boolean({
            description: messages.getMessage("packagefilterDescription"),
            char: "p",
            required: false
        }),
        showall: command_1.flags.boolean({
            char: "s",
            description: messages.getMessage("showallDescription"),
            required: false
        }),
        format: command_1.flags.enum({
            required: false,
            char: "f",
            description: messages.getMessage("formatDescription"),
            options: ["json", "csv"],
            default: "json"
        }),
        output: command_1.flags.string({
            char: "o",
            description: messages.getMessage("outputDescription"),
            required: true
        }),
        loglevel: command_1.flags.enum({
            description: messages.getMessage("loglevelDescription"),
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    // Comment this out if your command does not require an org username
    Tree.requiresUsername = true;
    return Tree;
})();
exports.default = Tree;
//# sourceMappingURL=package.js.map