"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const rimraf = __importStar(require("rimraf"));
const spawn = require("child-process-promise").spawn;
const zipDirectory_1 = require("../../../../utils/zipDirectory");
const checkDeploymentStatus_1 = require("../../../../utils/checkDeploymentStatus");
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../../sfpowerkit");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "connectedapp_create");
let Create = /** @class */ (() => {
    class Create extends command_1.SfdxCommand {
        // Comment this out if your command does not support a hub org username
        // protected static supportsDevhubUsername = true;
        // Set this to true if your command requires a project workspace; 'requiresProject' is false by default
        //protected static requiresProject = true;
        async run() {
            rimraf.sync("temp_sfpowerkit");
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            await this.org.refreshAuth();
            //Connect to the org
            const conn = this.org.getConnection();
            const apiversion = await conn.retrieveMaxApiVersion();
            // this.org is guaranteed because requiresUsername=true, as opposed to supportsUsername
            const username = this.org.getUsername();
            const pathToCertificate = this.flags.pathtocertificate.valueOf();
            this.connectedapp_email = this.flags.email;
            this.connectedapp_label = this.flags.name;
            var certificate = fs.readFileSync(pathToCertificate).toString();
            var textblock = certificate.split("\n");
            textblock.splice(0, 1);
            textblock.splice(-2, 1);
            certificate = textblock.join("\n");
            certificate = certificate.replace(/(\r\n|\n|\r)/gm, "");
            this.connectedapp_certificate = certificate;
            this.connectedapp_consumerKey = this.createConsumerKey();
            var connectedApp_metadata = `<?xml version="1.0" encoding="UTF-8"?>
     <ConnectedApp xmlns="http://soap.sforce.com/2006/04/metadata">
         <contactEmail>${this.connectedapp_email}</contactEmail>
         <label>${this.connectedapp_label}</label>
         <oauthConfig>
             <callbackUrl>http://localhost:1717/OauthRedirect</callbackUrl>
             <certificate>${this.connectedapp_certificate}</certificate>
             <consumerKey>${this.connectedapp_consumerKey}</consumerKey>
             <scopes>Api</scopes>
             <scopes>Web</scopes>
             <scopes>RefreshToken</scopes>
         </oauthConfig>
     </ConnectedApp>`;
            var package_xml = `<?xml version="1.0" encoding="UTF-8"?>
     <Package xmlns="http://soap.sforce.com/2006/04/metadata">
         <types>
             <members>*</members>
             <name>ConnectedApp</name>
         </types>
         <version>${apiversion}</version>
     </Package>`;
            let targetmetadatapath = "temp_sfpowerkit/mdapi/connectedApps/" +
                this.connectedapp_label +
                ".connectedApp-meta.xml";
            fs.outputFileSync(targetmetadatapath, connectedApp_metadata);
            let targetpackagepath = "temp_sfpowerkit/mdapi/package.xml";
            fs.outputFileSync(targetpackagepath, package_xml);
            var zipFile = "temp_sfpowerkit/package.zip";
            await zipDirectory_1.zipDirectory("temp_sfpowerkit/mdapi", zipFile);
            //Deploy Rule
            conn.metadata.pollTimeout = 300;
            let deployId;
            var zipStream = fs.createReadStream(zipFile);
            await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                if (error) {
                    return console.error(error);
                }
                deployId = result;
            });
            this.ux.log(`Deploying Connected App with ID  ${deployId.id}  to ${this.org.getUsername()}`);
            let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
            if (!metadata_deploy_result.success)
                throw new core_1.SfdxError(`Unable to deploy the Connected App : ${metadata_deploy_result.details["componentFailures"]["problem"]}`);
            this.ux.log(`Connected App Deployed`);
            rimraf.sync("temp_sfpowerkit");
            return { "connectedapp.consumerkey": this.connectedapp_consumerKey };
        }
        createConsumerKey() {
            var text = "";
            var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789.";
            for (var i = 0; i < 32; i++)
                text += possible.charAt(Math.floor(Math.random() * possible.length));
            return text;
        }
    }
    Create.description = messages.getMessage("commandDescription");
    Create.examples = [
        `$ sfdx sfpowerkit:org:connectedapp:create -u myOrg@example.com -n AzurePipelines -c id_rsa -e azlam.salamm@invalid.com
  Created Connected App AzurePipelines in Target Org
  `
    ];
    Create.flagsConfig = {
        name: command_1.flags.string({
            required: true,
            char: "n",
            description: messages.getMessage("nameFlagDescription")
        }),
        pathtocertificate: command_1.flags.filepath({
            required: true,
            char: "c",
            description: messages.getMessage("certificateFlagDescription")
        }),
        email: command_1.flags.email({
            required: true,
            char: "e",
            description: messages.getMessage("emailFlagDescription")
        }),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    // Comment this out if your command does not require an org username
    Create.requiresUsername = true;
    return Create;
})();
exports.default = Create;
//# sourceMappingURL=create.js.map