"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = __importStar(require("rimraf"));
const core_1 = require("@salesforce/core");
const xml2js = __importStar(require("xml2js"));
const util = __importStar(require("util"));
// tslint:disable-next-line:ordered-imports
var jsforce = require("jsforce");
var path = require("path");
const checkRetrievalStatus_1 = require("../../../../utils/checkRetrievalStatus");
const checkDeploymentStatus_1 = require("../../../../utils/checkDeploymentStatus");
const extract_1 = require("../../../../utils/extract");
const zipDirectory_1 = require("../../../../utils/zipDirectory");
const sfpowerkit_1 = require("../../../../sfpowerkit");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "duplicaterule_activate");
let Activate = /** @class */ (() => {
    class Activate extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            rimraf.sync("temp_sfpowerkit");
            //Connect to the org
            await this.org.refreshAuth();
            const conn = this.org.getConnection();
            const apiversion = await conn.retrieveMaxApiVersion();
            let retrieveRequest = {
                apiVersion: apiversion
            };
            //Retrieve Duplicate Rule
            retrieveRequest["singlePackage"] = true;
            retrieveRequest["unpackaged"] = {
                types: { name: "DuplicateRule", members: this.flags.name }
            };
            conn.metadata.pollTimeout = 60;
            let retrievedId;
            await conn.metadata.retrieve(retrieveRequest, function (error, result) {
                if (error) {
                    return console.error(error);
                }
                retrievedId = result.id;
            });
            let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId, !this.flags.json);
            if (!metadata_retrieve_result.zipFile)
                throw new core_1.SfdxError("Unable to find the requested Duplicate Rule");
            //Extract Duplicate Rule
            var zipFileName = "temp_sfpowerkit/unpackaged.zip";
            fs.mkdirSync("temp_sfpowerkit");
            fs.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
                encoding: "base64"
            });
            await extract_1.extract(`./temp_sfpowerkit/unpackaged.zip`, "temp_sfpowerkit");
            fs.unlinkSync(zipFileName);
            let resultFile = `temp_sfpowerkit/duplicateRules/${this.flags.name}.duplicateRule`;
            if (fs.existsSync(path.resolve(resultFile))) {
                const parser = new xml2js.Parser({ explicitArray: false });
                const parseString = util.promisify(parser.parseString);
                let retrieved_duplicaterule = await parseString(fs.readFileSync(path.resolve(resultFile)));
                this.ux.log(`Retrieved Duplicate Rule  with label : ${retrieved_duplicaterule.DuplicateRule.masterLabel}`);
                //Do Nothing if its already Active
                if (retrieved_duplicaterule.DuplicateRule.isActive === "true") {
                    this.ux.log("Already Active, exiting");
                    return 1;
                }
                //Deactivate Rule
                this.ux.log(`Preparing Activation`);
                retrieved_duplicaterule.DuplicateRule.isActive = "true";
                let builder = new xml2js.Builder();
                var xml = builder.buildObject(retrieved_duplicaterule);
                fs.writeFileSync(resultFile, xml);
                var zipFile = "temp_sfpowerkit/package.zip";
                await zipDirectory_1.zipDirectory("temp_sfpowerkit", zipFile);
                //Deploy Rule
                conn.metadata.pollTimeout = 300;
                let deployId;
                var zipStream = fs.createReadStream(zipFile);
                await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                    if (error) {
                        return console.error(error);
                    }
                    deployId = result;
                });
                this.ux.log(`Deploying Activated Rule with ID  ${deployId.id}  to ${this.org.getUsername()}`);
                let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
                if (!metadata_deploy_result.success)
                    throw new core_1.SfdxError(`Unable to deploy the Activated rule : ${metadata_deploy_result.details["componentFailures"]["problem"]}`);
                this.ux.log(`Duplicate Rule ${retrieved_duplicaterule.DuplicateRule.masterLabel} Activated`);
                return 0;
            }
            else {
                throw new core_1.SfdxError("Duplicate Rule not found in the org");
            }
        }
    }
    Activate.description = messages.getMessage("commandDescription");
    Activate.examples = [
        `$ sfdx sfpowerkit:org:duplicaterule:Activate -n Account.CRM_Account_Rule_1 -u sandbox
    Polling for Retrieval Status
    Retrieved Duplicate Rule  with label : CRM Account Rule 2
    Preparing Activation
    Deploying Activated Rule with ID  0Af4Y000003OdTWSA0
    Polling for Deployment Status
    Polling for Deployment Status
    Duplicate Rule CRM Account Rule 2 Activated
  `
    ];
    Activate.flagsConfig = {
        name: command_1.flags.string({
            required: true,
            char: "n",
            description: messages.getMessage("nameFlagDescription")
        }),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    // Comment this out if your command does not require an org username
    Activate.requiresUsername = true;
    return Activate;
})();
exports.default = Activate;
//# sourceMappingURL=activate.js.map