"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../../sfpowerkit");
const profileDiff_1 = __importDefault(require("../../../../impl/source/profiles/profileDiff"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "org_profile_diff");
let Diff = /** @class */ (() => {
    class Diff extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            const outputFolder = this.flags.output;
            const sourceusername = this.flags.sourceusername;
            let profileList = this.flags.profilelist;
            if (!profileList || profileList.length === 0) {
                if (sourceusername && !outputFolder) {
                    throw new Error("Output folder is required");
                }
            }
            let profileDiff = new profileDiff_1.default(profileList, sourceusername, this.org, outputFolder);
            let output = profileDiff.diff().then(() => {
                return profileDiff.output;
            });
            let outputData = await output;
            return outputData;
        }
    }
    Diff.description = messages.getMessage("commandDescription");
    Diff.examples = [
        `$ sfdx sfpowerkit:org:profile:diff --profilelist profilenames --targetusername username (Compare liste profiles path against target org)`,
        `$ sfdx sfpowerkit:org:profile:diff --targetusername username (compare all profile in the project against the target org)`,
        `$ sfdx sfpowerkit:org:profile:diff --sourceusername sourcealias --targetusername username (compare all profile in the source org against the target org)`
    ];
    Diff.flagsConfig = {
        profilelist: command_1.flags.array({
            char: "p",
            description: messages.getMessage("profileListFlagDescription"),
            required: false,
            map: (n) => n.trim()
        }),
        sourceusername: command_1.flags.string({
            char: "s",
            description: messages.getMessage("sourceUsernameDescription"),
            required: false
        }),
        output: command_1.flags.string({
            char: "d",
            description: messages.getMessage("outputFolderDescription"),
            required: false
        }),
        apiversion: command_1.flags.builtin(),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    Diff.result = {
        tableColumnData: {
            columns: [
                { key: "status", label: "Status" },
                { key: "metadataType", label: "Type" },
                { key: "componentName", label: "Component Name" },
                { key: "path", label: "Path" }
            ]
        },
        display() {
            if (Array.isArray(this.data) && this.data.length) {
                this.ux.table(this.data, this.tableColumnData);
            }
        }
    };
    Diff.requiresUsername = true;
    return Diff;
})();
exports.default = Diff;
//# sourceMappingURL=diff.js.map