"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const rimraf = __importStar(require("rimraf"));
const core_1 = require("@salesforce/core");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "sandbox_create");
let Create = /** @class */ (() => {
    class Create extends command_1.SfdxCommand {
        async run() {
            rimraf.sync("temp_sfpowerkit");
            await this.hubOrg.refreshAuth();
            const conn = this.hubOrg.getConnection();
            this.flags.apiversion =
                this.flags.apiversion || (await conn.retrieveMaxApiVersion());
            const uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/sobjects/SandboxInfo/`;
            this.ux.log(`${this.flags.apexclass}  ${this.flags.clonefrom} `);
            var sourceSandboxId = "";
            var result;
            if (this.flags.clonefrom) {
                const sourceSandboxId = await this.getSandboxId(conn, this.flags.clonefrom);
                result = await request_promise_native_1.default({
                    method: "post",
                    uri,
                    headers: {
                        Authorization: `Bearer ${conn.accessToken}`
                    },
                    body: {
                        AutoActivate: "true",
                        SandboxName: `${this.flags.name}`,
                        Description: `${this.flags.description}`,
                        ApexClassId: `${this.flags.apexclass}`,
                        SourceId: sourceSandboxId
                    },
                    json: true
                });
            }
            else {
                result = await request_promise_native_1.default({
                    method: "post",
                    uri,
                    headers: {
                        Authorization: `Bearer ${conn.accessToken}`
                    },
                    body: {
                        AutoActivate: "true",
                        SandboxName: `${this.flags.name}`,
                        Description: `${this.flags.description}`,
                        LicenseType: `${this.flags.licensetype}`,
                        ApexClassId: `${this.flags.apexclass}`
                    },
                    json: true
                });
            }
            if (result.success) {
                this.ux.log(`Successfully Enqueued Creation of Sandbox`);
                this.ux.log(result);
            }
            else {
                throw new core_1.SfdxError("Unable to Create sandbox");
            }
            if (this.flags.outputfile) {
                await fs.outputJSON(this.flags.outputfile, result);
            }
            rimraf.sync("temp_sfpowerkit");
            return result;
        }
        async getSandboxId(conn, name) {
            const query_uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/query?q=SELECT+Id,SandboxName+FROM+SandboxInfo+WHERE+SandboxName+in+('${name}')`;
            // this.ux.log(`Query URI ${query_uri}`);
            const sandbox_query_result = await request_promise_native_1.default({
                method: "get",
                url: query_uri,
                headers: {
                    Authorization: `Bearer ${conn.accessToken}`
                },
                json: true
            });
            // this.ux.logJson(sandbox_query_result);
            if (sandbox_query_result.records[0] == undefined)
                throw new core_1.SfdxError(`Unable to continue, Please check your sandbox name: ${name}`);
            this.ux.log(`Fetched Sandbox Id for sandbox  ${name}  is ${sandbox_query_result.records[0].Id}`);
            return sandbox_query_result.records[0].Id;
        }
    }
    Create.description = messages.getMessage("commandDescription");
    Create.examples = [
        `$ sfdx sfpowerkit:org:sandbox:create -d Testsandbox -l DEVELOPER -n test2 -u myOrg@example.com
  Successfully Enqueued Creation of Sandbox
  `
    ];
    Create.flagsConfig = {
        name: command_1.flags.string({
            required: true,
            char: "n",
            description: messages.getMessage("nameFlagDescription")
        }),
        description: command_1.flags.string({
            required: true,
            char: "d",
            description: messages.getMessage("descriptionFlagDescription")
        }),
        licensetype: command_1.flags.string({
            required: true,
            char: "l",
            options: ["DEVELOPER", "DEVELOPER_PRO", "PARTIAL", "FULL"],
            description: messages.getMessage("licenseFlagDescription")
        }),
        apexclass: command_1.flags.string({
            required: false,
            char: "a",
            default: "",
            description: messages.getMessage("apexClassFlagDescription")
        }),
        clonefrom: command_1.flags.string({
            required: false,
            char: "f",
            default: "",
            description: messages.getMessage("cloneFromFlagDescripton")
        })
    };
    // Comment this out if your command does not require a hub org username
    Create.requiresDevhubUsername = true;
    return Create;
})();
exports.default = Create;
//# sourceMappingURL=create.js.map