"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const request_promise_native_1 = __importDefault(require("request-promise-native"));
const rimraf = __importStar(require("rimraf"));
const core_1 = require("@salesforce/core");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "sandbox_info");
let Info = /** @class */ (() => {
    class Info extends command_1.SfdxCommand {
        async run() {
            rimraf.sync("temp_sfpowerkit");
            // const conn = await Connection.create({
            //   authInfo: await AuthInfo.create({ username: `${this.org.getUsername()}` })
            // });
            await this.hubOrg.refreshAuth();
            const conn = this.hubOrg.getConnection();
            this.flags.apiversion =
                this.flags.apiversion || (await conn.retrieveMaxApiVersion());
            var result = await this.getSandboxInfo(conn, this.flags.name);
            if (this.flags.outputfile) {
                await fs.outputJSON(this.flags.outputfile, result);
            }
            this.ux.log(`Successfully Retrieved Sandbox Details`);
            if (!this.flags.json)
                this.ux.logJson(result);
            rimraf.sync("temp_sfpowerkit");
            return result;
        }
        async getSandboxInfo(conn, name) {
            var query_uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/query?q=SELECT+Id,SandboxName+FROM+SandboxProcess+WHERE+SandboxName+in+('${name}')+ORDER+BY+EndDate+DESC`;
            //this.ux.log(`Query URI ${query_uri}`);
            const sandbox_query_result = await request_promise_native_1.default({
                method: "get",
                url: query_uri,
                headers: {
                    Authorization: `Bearer ${conn.accessToken}`
                },
                json: true
            });
            if (sandbox_query_result.records[0] == undefined)
                throw new core_1.SfdxError(`Unable to find a sandbox with name: ${name}`);
            var result = await this.processSandboxInfo(sandbox_query_result.records, conn, this.flags.showonlylatest);
            return result;
        }
        async processSandboxInfo(sandboxRecords, conn, isShowOnlyLatest) {
            var result = [];
            for (const item of sandboxRecords) {
                var output = await this.getDetailedSandboxInfo(item.attributes.url, conn);
                result.push(output);
                if (isShowOnlyLatest)
                    break;
            }
            return result;
        }
        async getDetailedSandboxInfo(sandboxInfoUl, conn) {
            const query_uri = `${conn.instanceUrl}${sandboxInfoUl}`;
            //this.ux.log(`Query URI ${query_uri}`);
            const sandbox_query_result = await request_promise_native_1.default({
                method: "get",
                url: query_uri,
                headers: {
                    Authorization: `Bearer ${conn.accessToken}`
                },
                json: true
            });
            return sandbox_query_result;
        }
    }
    Info.description = messages.getMessage("commandDescription");
    Info.examples = [
        `$ sfdx sfpowerkit:org:sandbox:info -n test2  -v produser@example.com 
  Successfully Enqueued Refresh of Sandbox
  `
    ];
    Info.flagsConfig = {
        name: command_1.flags.string({
            required: true,
            char: "n",
            description: messages.getMessage("nameFlagDescription")
        }),
        showonlylatest: command_1.flags.boolean({
            required: false,
            char: "s",
            default: false,
            description: messages.getMessage("showOnlyLatestFlagDescription")
        })
    };
    // Comment this out if your command does not require a hub org username
    Info.requiresDevhubUsername = true;
    return Info;
})();
exports.default = Info;
//# sourceMappingURL=info.js.map