"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../../sfpowerkit");
const packageInfo_1 = __importDefault(require("../../../../impl/package/version/packageInfo"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "package_info");
let Info = /** @class */ (() => {
    class Info extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            await this.org.refreshAuth();
            const conn = this.org.getConnection();
            this.flags.apiversion =
                this.flags.apiversion || (await conn.retrieveMaxApiVersion());
            let packageInfoImpl = new packageInfo_1.default(conn, this.flags.apiversion, this.flags.json);
            const result = (await packageInfoImpl.getPackages());
            result.sort((a, b) => (a.packageName > b.packageName ? 1 : -1));
            this.ux.table(result, [
                "packageName",
                "packageNamespacePrefix",
                "packageVersionNumber",
                "packageVersionId",
                "allowedLicenses",
                "usedLicenses",
                "expirationDate",
                "status"
            ]);
            return result;
        }
    }
    Info.description = messages.getMessage("commandDescription");
    Info.examples = [
        `$ sfdx sfpowerkit:package:version:info -u myOrg@example.com `
    ];
    Info.flagsConfig = {
        apiversion: command_1.flags.builtin({
            description: messages.getMessage("apiversion")
        }),
        loglevel: command_1.flags.enum({
            description: messages.getMessage("loglevel"),
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    // Comment this out if your command does not require an org username
    Info.requiresUsername = true;
    return Info;
})();
exports.default = Info;
//# sourceMappingURL=info.js.map