"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../sfpowerkit");
const poolFetchImpl_1 = __importDefault(require("../../../impl/pool/scratchorg/poolFetchImpl"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "scratchorg_poolFetch");
let Fetch = /** @class */ (() => {
    class Fetch extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel("DEBUG", false);
            await this.hubOrg.refreshAuth();
            const hubConn = this.hubOrg.getConnection();
            this.flags.apiversion =
                this.flags.apiversion || (await hubConn.retrieveMaxApiVersion());
            let fetchImpl = new poolFetchImpl_1.default(this.hubOrg, this.flags.apiversion, this.flags.tag, this.flags.mypool);
            let result = await fetchImpl.execute();
            if (!this.flags.json) {
                this.ux.log(`======== Scratch org details ========`);
                let list = [];
                for (let [key, value] of Object.entries(result)) {
                    if (value) {
                        list.push({ key: key, value: value });
                    }
                }
                this.ux.table(list, ["key", "value"]);
            }
            return JSON.stringify(result);
        }
    }
    Fetch.description = messages.getMessage("commandDescription");
    Fetch.requiresDevhubUsername = true;
    Fetch.examples = [
        `$ sfdx sfpowerkit:pool:fetch -t core `,
        `$ sfdx sfpowerkit:pool:fetch -t core -v devhub`,
        `$ sfdx sfpowerkit:pool:fetch -t core -v devhub -m`
    ];
    Fetch.flagsConfig = {
        tag: command_1.flags.string({
            char: "t",
            description: messages.getMessage("tagDescription"),
            required: true
        }),
        mypool: command_1.flags.boolean({
            char: "m",
            description: messages.getMessage("mypoolDescription"),
            required: false
        })
    };
    return Fetch;
})();
exports.default = Fetch;
//# sourceMappingURL=fetch.js.map