"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const diffImpl_1 = __importDefault(require("../../../impl/project/diff/diffImpl"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const core_1 = require("@salesforce/core");
const rimraf = __importStar(require("rimraf"));
const fsextra = __importStar(require("fs-extra"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_diff");
let Diff = /** @class */ (() => {
    class Diff extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            const outputFolder = this.flags.output;
            const revisionfrom = this.flags.revisionfrom;
            const revisionto = this.flags.revisionto;
            let diffUtils = new diffImpl_1.default(revisionfrom, revisionto, this.flags.generatedestructive, this.flags.bypass);
            let diffOutput = await diffUtils.build(outputFolder, this.flags.packagedirectories, this.flags.apiversion);
            let errors = diffOutput.filter(element => {
                return element["action"] === "ERROR";
            });
            if (errors && errors.length > 0) {
                this.ux.log("ERRORS");
                this.ux.table(errors, {
                    columns: [
                        { key: "path", label: "Path" },
                        { key: "message", label: "Error Message" }
                    ]
                });
                rimraf.sync(outputFolder);
                if (this.flags.json) {
                    //In case of error, the diff output is still printed in the output folder
                    if (fsextra.existsSync(outputFolder) == false) {
                        fsextra.mkdirSync(outputFolder);
                        core_1.fs.writeJson(path.join(outputFolder, "diff.json"), diffOutput);
                    }
                }
                throw new Error("Error parsing diff.");
            }
            else {
                core_1.fs.writeJson(path.join(outputFolder, "diff.json"), diffOutput);
            }
            return diffOutput;
        }
    }
    Diff.description = messages.getMessage("commandDescription");
    Diff.examples = [
        `$ sfdx sfpowerkit:project:diff --diffFile DiffFileName --encoding EncodingOfFile --output OutputFolder`,
        `$ sfdx sfpowerkit:project:diff --revisionfrom revisionfrom --revisionto revisionto --output OutputFolder
   `
    ];
    Diff.flagsConfig = {
        revisionfrom: command_1.flags.string({
            char: "r",
            description: messages.getMessage("revisionFromDescription"),
            required: true
        }),
        revisionto: command_1.flags.string({
            char: "t",
            description: messages.getMessage("revisionToDescription"),
            required: false
        }),
        output: command_1.flags.string({
            char: "d",
            description: messages.getMessage("outputFolderDescription"),
            required: true
        }),
        generatedestructive: command_1.flags.boolean({
            char: "x",
            description: messages.getMessage("generativeDestructiveManifestDescription"),
            required: false
        }),
        bypass: command_1.flags.array({
            required: false,
            char: "b",
            description: messages.getMessage("itemsToBypass")
        }),
        packagedirectories: command_1.flags.array({
            required: false,
            char: "p",
            description: messages.getMessage("packagedirectories")
        }),
        apiversion: command_1.flags.builtin({
            description: messages.getMessage("apiversion")
        }),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    Diff.result = {
        tableColumnData: {
            columns: [
                { key: "action", label: "Action (Deploy/Delete)" },
                { key: "metadataType", label: "Type" },
                { key: "componentName", label: "Component Name" },
                { key: "path", label: "Path" }
            ]
        },
        display() {
            if (Array.isArray(this.data) && this.data.length) {
                this.ux.table(this.data.filter(element => {
                    return element["action"] !== "ERROR";
                }), this.tableColumnData);
            }
        }
    };
    Diff.requiresUsername = false;
    Diff.requiresProject = true;
    return Diff;
})();
exports.default = Diff;
//# sourceMappingURL=diff.js.map