"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const xmlUtil_1 = __importDefault(require("../../../../utils/xmlUtil"));
const getDefaults_1 = __importDefault(require("../../../../utils/getDefaults"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_manifest_diff");
let Diff = /** @class */ (() => {
    class Diff extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            this.flags.apiversion =
                this.flags.apiversion || getDefaults_1.default.getApiVersion();
            if (this.flags.json) {
                this.flags.format = "json";
            }
            let sourceXml = await this.processMainfest(this.flags.sourcepath);
            let targetXml = await this.processMainfest(this.flags.targetpath);
            let itemsAddedInTarget = this.compareXML(sourceXml, targetXml);
            let itemsRemovedInTarget = this.compareXML(targetXml, sourceXml);
            this.output = [];
            if (itemsAddedInTarget || itemsRemovedInTarget) {
                this.addItemsToOutput(itemsAddedInTarget, "Added in Target");
                this.addItemsToOutput(itemsRemovedInTarget, "Removed in Target");
                this.output.sort(function (a, b) {
                    if (a.type < b.type) {
                        return -1;
                    }
                    else if (a.type > b.type) {
                        return 1;
                    }
                    // names must be equal
                    return 0;
                });
                if (this.flags.format === "xml") {
                    this.createpackagexml(itemsAddedInTarget);
                }
                else if (this.flags.format === "csv") {
                    this.generateCSVOutput(this.output);
                }
                else {
                    fs.writeFileSync(`${this.flags.output}/package.json`, JSON.stringify(this.output));
                }
            }
            return this.output;
        }
        async processMainfest(pathToManifest) {
            let output = new Map();
            if (fs.existsSync(path.resolve(pathToManifest)) &&
                path.extname(pathToManifest) == ".xml") {
                let package_xml = await xmlUtil_1.default.xmlToJSON(pathToManifest);
                let metadataTypes = package_xml.Package.types;
                if (metadataTypes.constructor === Array) {
                    metadataTypes.forEach(type => {
                        if (type.members !== "*") {
                            output.set(type.name, type.members.constructor === Array ? type.members : [type.members]);
                        }
                    });
                }
                else {
                    if (metadataTypes.members !== "*") {
                        output.set(metadataTypes.name, metadataTypes.members.constructor === Array
                            ? metadataTypes.members
                            : [metadataTypes.members]);
                    }
                }
            }
            else {
                throw new Error(`Error : ${pathToManifest} is not valid package.xml`);
            }
            return output;
        }
        compareXML(sourceXml, targetXml) {
            let metadataTypes = [];
            if (sourceXml && targetXml) {
                for (let key of targetXml.keys()) {
                    if (sourceXml.has(key)) {
                        const diffout = this.getdiffList(sourceXml.get(key), targetXml.get(key));
                        if (diffout) {
                            metadataTypes.push({ name: key, members: diffout });
                        }
                    }
                    else {
                        metadataTypes.push({ name: key, members: targetXml.get(key) });
                    }
                }
            }
            return metadataTypes;
        }
        getdiffList(from, to) {
            let output = [];
            to.forEach(item => {
                if (!from.includes(item)) {
                    output.push(item);
                }
            });
            return output;
        }
        addItemsToOutput(itemsToProcess, status) {
            itemsToProcess.forEach(metadataType => {
                for (let item of metadataType.members) {
                    this.output.push({
                        status: status,
                        type: metadataType.name,
                        member: item
                    });
                }
            });
        }
        createpackagexml(manifest) {
            let package_xml = {
                Package: {
                    $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
                    types: manifest,
                    version: this.flags.apiversion
                }
            };
            fs.outputFileSync(`${this.flags.output}/package.xml`, xmlUtil_1.default.jSONToXML(package_xml));
        }
        generateCSVOutput(output) {
            let newLine = "\r\n";
            let result = "status,type,member" + newLine;
            output.forEach(element => {
                result = `${result}${element.status},${element.type},${element.member}${newLine}`;
            });
            fs.writeFileSync(`${this.flags.output}/package.csv`, result);
        }
    }
    Diff.description = messages.getMessage("commandDescription");
    Diff.examples = [
        `$ sfdx sfpowerkit:project:manifest:diff -s source/package.xml -t target/package.xml -d output`
    ];
    Diff.flagsConfig = {
        sourcepath: command_1.flags.string({
            required: true,
            char: "s",
            description: messages.getMessage("sourcepathFlagDescription")
        }),
        targetpath: command_1.flags.string({
            required: true,
            char: "t",
            description: messages.getMessage("targetpathFlagDescription")
        }),
        output: command_1.flags.string({
            required: true,
            char: "d",
            description: messages.getMessage("outputFlagDescription")
        }),
        apiversion: command_1.flags.builtin({
            description: messages.getMessage("apiversion")
        }),
        format: command_1.flags.enum({
            required: false,
            char: "f",
            description: messages.getMessage("formatFlagDescription"),
            options: ["json", "csv", "xml"],
            default: "json"
        }),
        loglevel: command_1.flags.enum({
            description: messages.getMessage("loglevel"),
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    return Diff;
})();
exports.default = Diff;
//# sourceMappingURL=diff.js.map