"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const xmlUtil_1 = __importDefault(require("../../../../utils/xmlUtil"));
const getDefaults_1 = __importDefault(require("../../../../utils/getDefaults"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_manifest_merge");
let Merge = /** @class */ (() => {
    class Merge extends command_1.SfdxCommand {
        async run() {
            this.output = new Map();
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            this.flags.apiversion =
                this.flags.apiversion || getDefaults_1.default.getApiVersion();
            let paths = [];
            if (this.flags.path.constructor === Array) {
                paths = this.flags.path;
            }
            else {
                paths.push(this.flags.path);
            }
            for (const dir of paths) {
                if (fs.existsSync(path.resolve(dir)) && path.extname(dir) == ".xml") {
                    await this.processMainfest(dir);
                }
                else {
                    throw new Error(`Error : ${dir} is not valid package.xml`);
                }
            }
            let metadataTypes = [];
            for (let [key, value] of this.output) {
                metadataTypes.push({ name: key, members: value });
            }
            if (metadataTypes) {
                this.createpackagexml(metadataTypes);
            }
            if (!this.flags.json) {
                let tableout = [];
                metadataTypes.forEach(metadataType => {
                    for (let item of metadataType.members) {
                        tableout.push({ type: metadataType.name, member: item });
                    }
                });
                this.ux.table(tableout, ["type", "member"]);
            }
            return metadataTypes;
        }
        async processMainfest(dir) {
            let package_xml = await xmlUtil_1.default.xmlToJSON(dir);
            let metadataTypes = package_xml.Package.types;
            if (metadataTypes.constructor === Array) {
                for (const item of metadataTypes) {
                    if (item.members.constructor === Array) {
                        this.setOutput(item.name, item.members);
                    }
                    else {
                        this.setOutput(item.name, [item.members]);
                    }
                }
            }
            else {
                if (metadataTypes.members.constructor === Array) {
                    this.setOutput(metadataTypes.name, metadataTypes.members);
                }
                else {
                    this.setOutput(metadataTypes.name, [metadataTypes.members]);
                }
            }
        }
        setOutput(key, values) {
            let currentItems = this.output.get(key) || [];
            values.forEach(item => {
                if (!currentItems.includes(item)) {
                    currentItems.push(item);
                }
            });
            this.output.set(key, currentItems);
        }
        createpackagexml(manifest) {
            let package_xml = {
                Package: {
                    $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
                    types: manifest,
                    version: this.flags.apiversion
                }
            };
            fs.outputFileSync(`${this.flags.manifest}/package.xml`, xmlUtil_1.default.jSONToXML(package_xml));
        }
    }
    Merge.description = messages.getMessage("commandDescription");
    Merge.examples = [
        `$ sfdx sfpowerkit:project:manifest:merge -p project1/path/to/package.xml -d result/package.xml\n` +
            `$ sfdx sfpowerkit:project:manifest:merge -p project1/path/to/package.xml,project2/path/to/package.xml -d result/package.xml`
    ];
    Merge.flagsConfig = {
        path: command_1.flags.array({
            required: true,
            char: "p",
            description: messages.getMessage("pathFlagDescription")
        }),
        manifest: command_1.flags.string({
            required: true,
            char: "d",
            description: messages.getMessage("manifestFlagDescription")
        }),
        apiversion: command_1.flags.builtin({
            description: messages.getMessage("apiversion")
        }),
        loglevel: command_1.flags.enum({
            description: messages.getMessage("loglevel"),
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    return Merge;
})();
exports.default = Merge;
//# sourceMappingURL=merge.js.map