"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const sfpowerkit_1 = require("../../../sfpowerkit");
const orgDiffImpl_1 = __importDefault(require("../../../impl/project/orgdiff/orgDiffImpl"));
const core_1 = require("@salesforce/core");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "project_orgdiff");
let OrgDiff = /** @class */ (() => {
    class OrgDiff extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setUx(this.ux);
            this.ux.startSpinner("Running...");
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            let filesOrFolders = this.flags.filesorfolders;
            let orgDiff = new orgDiffImpl_1.default(filesOrFolders, this.org, !this.flags.noconflictmarkers);
            let output = await orgDiff.orgDiff();
            this.ux.stopSpinner("Completed");
            if (!this.flags.outputformat || this.flags.outputformat == "json") {
                core_1.fs.writeJson("orgdiff.json", output);
            }
            else if (this.flags.outputformat == "csv") {
                await this.generateCSVOutput(output);
            }
            return output;
        }
        async generateCSVOutput(result) {
            let newLine = "\r\n";
            let output = "status,metadataType,componentName,path" + newLine;
            result.forEach(element => {
                output = `${output}${element.status},${element.metadataType},${element.componentName},${element.path}${newLine}`;
            });
            core_1.fs.writeFile("orgdiff.csv", output);
        }
    }
    OrgDiff.description = messages.getMessage("commandDescription");
    OrgDiff.examples = [
        `$ sfdx sfpowerkit:project:orgdiff --folder directory --noconflictmarkers --targetusername sandbox`,
        `$ sfdx sfpowerkit:project:orgdiff  --filename fileName --targetusername sandbox`
    ];
    OrgDiff.flagsConfig = {
        filesorfolders: command_1.flags.array({
            char: "f",
            description: messages.getMessage("filesOrFoldersFlagDescription"),
            required: true,
            map: (f) => f.trim()
        }),
        noconflictmarkers: command_1.flags.boolean({
            char: "c",
            description: messages.getMessage("noConflictMarkersDescription"),
            required: false
        }),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        }),
        outputformat: command_1.flags.enum({
            required: false,
            char: "o",
            description: messages.getMessage("outputFormatFlagDescription"),
            options: ["json", "csv"]
        })
    };
    OrgDiff.result = {
        tableColumnData: {
            columns: [
                { key: "status", label: "Status" },
                { key: "metadataType", label: "Type" },
                { key: "componentName", label: "Component Name" },
                { key: "path", label: "Path" }
            ]
        },
        display() {
            if (Array.isArray(this.data) && this.data.length) {
                this.ux.table(this.data, this.tableColumnData);
            }
        }
    };
    OrgDiff.requiresUsername = true;
    OrgDiff.requiresProject = true;
    return OrgDiff;
})();
exports.default = OrgDiff;
//# sourceMappingURL=orgdiff.js.map