"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const xmlUtil_1 = __importDefault(require("../../../../utils/xmlUtil"));
const getDefaults_1 = __importDefault(require("../../../../utils/getDefaults"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_customlabel_buildmanifest");
let Buildmanifest = /** @class */ (() => {
    class Buildmanifest extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            this.flags.apiversion =
                this.flags.apiversion || getDefaults_1.default.getApiVersion();
            let paths = [];
            if (this.flags.path.constructor === Array) {
                paths = this.flags.path;
            }
            else {
                paths.push(this.flags.path);
            }
            this.output = [];
            for (const element of paths) {
                if (fs.existsSync(path.resolve(element)) &&
                    (element.endsWith("CustomLabels.labels") ||
                        element.endsWith("CustomLabels.labels-meta.xml"))) {
                    await this.getlabels(element);
                }
                else {
                    throw new Error(`Error : ${element} is not valid custom label file`);
                }
            }
            this.flags.manifest = await this.validatepackagexml(this.flags.manifest);
            await this.setlabels(this.flags.manifest);
            if (!this.flags.json) {
                let result = [];
                for (let i = 0; i < this.output.length; i++) {
                    result.push({ sno: i + 1, label: this.output[i] });
                }
                this.ux.table(result, ["sno", "label"]);
            }
            return this.output;
        }
        setoutput(label) {
            if (!this.output.includes(label) && label !== "*") {
                this.output.push(label);
            }
        }
        async getlabels(labelpath) {
            let retrieved_customlabels = await xmlUtil_1.default.xmlToJSON(labelpath);
            let labels = retrieved_customlabels.CustomLabels.labels;
            if (labels.constructor === Array) {
                labels.forEach(label => {
                    this.setoutput(label.fullName);
                });
            }
            else {
                this.setoutput(labels.fullName);
            }
        }
        async validatepackagexml(manifest) {
            let fileOrFolder = path.normalize(manifest);
            if (fs.existsSync(fileOrFolder)) {
                let stats = fs.statSync(fileOrFolder);
                if (stats.isFile()) {
                    if (path.extname(fileOrFolder) != ".xml") {
                        throw new Error(`Error : ${fileOrFolder} is not valid package.xml`);
                    }
                    else {
                        await this.checklabelspackagexml(manifest);
                    }
                }
                else if (stats.isDirectory()) {
                    manifest = `${manifest}/package.xml`;
                    this.createpackagexml(manifest);
                }
            }
            else {
                manifest = manifest.endsWith(`package.xml`)
                    ? `${manifest}`
                    : `${manifest}/package.xml`;
                this.createpackagexml(manifest);
            }
            return manifest;
        }
        createpackagexml(manifest) {
            var package_xml = `<?xml version="1.0" encoding="UTF-8"?>
<Package xmlns="http://soap.sforce.com/2006/04/metadata">
  <types>
      <members>*</members>
      <name>CustomLabel</name>
  </types>
  <version>${this.flags.apiversion}</version>
</Package>`;
            fs.outputFileSync(manifest, package_xml);
        }
        async checklabelspackagexml(manifest) {
            let package_xml = await xmlUtil_1.default.xmlToJSON(manifest);
            let isLabelexist = false;
            if (package_xml.Package.types.constructor === Array) {
                for (const item of package_xml.Package.types) {
                    if (item.name === "CustomLabel") {
                        this.setlabelutil(item.members);
                        item.members = "*";
                        isLabelexist = true;
                        break;
                    }
                }
            }
            else if (package_xml.Package.types.name === "CustomLabel") {
                this.setlabelutil(package_xml.Package.types.members);
                package_xml.Package.types.members = "*";
                isLabelexist = true;
            }
            if (!isLabelexist) {
                let label = { name: "CustomLabel", members: "*" };
                package_xml.Package.types.push(label);
            }
            fs.outputFileSync(manifest, xmlUtil_1.default.jSONToXML(package_xml));
        }
        setlabelutil(members) {
            if (members.constructor === Array) {
                for (const label of members) {
                    this.setoutput(label);
                }
            }
            else {
                this.setoutput(members);
            }
        }
        async setlabels(manifest) {
            let package_xml = await xmlUtil_1.default.xmlToJSON(manifest);
            if (package_xml.Package.types.constructor === Array) {
                for (const item of package_xml.Package.types) {
                    if (item.name === "CustomLabel") {
                        item.members = this.output;
                        break;
                    }
                }
            }
            else if (package_xml.Package.types.name === "CustomLabel") {
                package_xml.Package.types.members = this.output;
            }
            fs.outputFileSync(manifest, xmlUtil_1.default.jSONToXML(package_xml));
        }
    }
    Buildmanifest.description = messages.getMessage("commandDescription");
    Buildmanifest.examples = [
        `$ sfdx sfpowerkit:source:customlabel:buildmanifest -p project1/path/to/customlabelfile.xml -x mdapiout/package.xml\n` +
            `$ sfdx sfpowerkit:source:customlabel:buildmanifest -p project1/path/to/customlabelfile.xml,project2/path/to/customlabelfile.xml -x mdapiout/package.xml`
    ];
    Buildmanifest.flagsConfig = {
        path: command_1.flags.array({
            required: true,
            char: "p",
            description: messages.getMessage("pathFlagDescription")
        }),
        manifest: command_1.flags.string({
            required: true,
            char: "x",
            description: messages.getMessage("manifestFlagDescription")
        }),
        apiversion: command_1.flags.builtin({
            description: messages.getMessage("apiversion")
        }),
        loglevel: command_1.flags.enum({
            description: messages.getMessage("loglevel"),
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    return Buildmanifest;
})();
exports.default = Buildmanifest;
//# sourceMappingURL=buildmanifest.js.map