"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const find_java_home_1 = __importDefault(require("find-java-home"));
const child_process_1 = require("child_process");
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const extract_1 = require("../../../utils/extract");
const util_1 = require("util");
const xml2js = __importStar(require("xml2js"));
const request = require("request");
const fs = require("fs");
const path = require("path");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "source_pmd");
let Pmd = /** @class */ (() => {
    class Pmd extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            if (util_1.isNullOrUndefined(this.flags.javahome)) {
                this.javahome = await this.findJavaHomeAsync();
                sfpowerkit_1.SFPowerkit.log(`Found Java Home at ${this.javahome}`, sfpowerkit_1.LoggerLevel.INFO);
            }
            //Download PMD
            let cache_directory = fileutils_1.default.getGlobalCacheDir();
            let pmd_cache_directory = path.join(cache_directory, "pmd");
            if (!fs.existsSync(path.join(pmd_cache_directory, `pmd-bin-${this.flags.version}`))) {
                sfpowerkit_1.SFPowerkit.log("Initiating Download of  PMD", sfpowerkit_1.LoggerLevel.INFO);
                if (!fs.existsSync(pmd_cache_directory))
                    fs.mkdirSync(pmd_cache_directory);
                await this.downloadPMD(this.flags.version, pmd_cache_directory);
                sfpowerkit_1.SFPowerkit.log(`Downloaded PMD ${this.flags.version}`, sfpowerkit_1.LoggerLevel.INFO);
                await extract_1.extract(path.join(pmd_cache_directory, "pmd.zip"), pmd_cache_directory);
                sfpowerkit_1.SFPowerkit.log(`Extracted PMD ${this.flags.version}`, sfpowerkit_1.LoggerLevel.INFO);
            }
            const pmdClassPath = path.join(pmd_cache_directory, `pmd-bin-${this.flags.version}`, "lib", "*");
            const pmdOutputPath = path.join(pmd_cache_directory, `pmd-bin-${this.flags.version}`, "sf-pmd-output.xml");
            //Directory to be scanned
            let packageDirectory = util_1.isNullOrUndefined(this.flags.directory)
                ? await sfpowerkit_1.SFPowerkit.getDefaultFolder()
                : this.flags.directory;
            //Default Ruleset
            let ruleset = util_1.isNullOrUndefined(this.flags.ruleset)
                ? path.join(__dirname, "..", "..", "..", "..", "resources", "pmd-ruleset.xml")
                : this.flags.ruleset;
            console.log(`PMD release ${this.flags.version}`);
            console.log(`Now analyzing ${packageDirectory}`);
            const pmdCmd = child_process_1.spawn(path.join(this.javahome, "bin", "java"), [
                "-cp",
                pmdClassPath,
                "net.sourceforge.pmd.PMD",
                "-l",
                "apex",
                "-d",
                packageDirectory,
                "-R",
                ruleset,
                "-f",
                "xml",
                "-r",
                pmdOutputPath
            ]);
            const pmdCmdForConsoleLogging = child_process_1.spawn(path.join(this.javahome, "bin", "java"), [
                "-cp",
                pmdClassPath,
                "net.sourceforge.pmd.PMD",
                "-l",
                "apex",
                "-d",
                packageDirectory,
                "-R",
                ruleset,
                "-f",
                this.flags.format,
                "-r",
                this.flags.report
            ]);
            //capture pmd errors
            let pmd_error;
            let pmd_output;
            pmdCmd.stderr.on("data", data => {
                pmd_error = data;
            });
            pmdCmd.stdout.on("data", data => {
                pmd_output = data;
            });
            pmdCmd.on("close", code => {
                if (code == 4 || code == 0) {
                    this.parseXmlReport(pmdOutputPath, packageDirectory);
                    if (!this.flags.supressoutput) {
                        let violations = fs.readFileSync(this.flags.report).toString();
                        sfpowerkit_1.SFPowerkit.log(violations, sfpowerkit_1.LoggerLevel.INFO);
                    }
                }
                else if (code == 1) {
                    sfpowerkit_1.SFPowerkit.log("PMD Exited with some exceptions ", sfpowerkit_1.LoggerLevel.INFO);
                    sfpowerkit_1.SFPowerkit.log(pmd_error.toString(), sfpowerkit_1.LoggerLevel.ERROR);
                }
            });
        }
        async findJavaHomeAsync() {
            return new Promise((resolve, reject) => {
                find_java_home_1.default({ allowJre: true }, (err, res) => {
                    if (err) {
                        return reject(err);
                    }
                    return resolve(res);
                });
            });
        }
        async downloadPMD(npm_package_pmd_version, pmd_chache_directory) {
            let file = fs.createWriteStream(path.join(pmd_chache_directory, "pmd.zip"));
            await new Promise((resolve, reject) => {
                let stream = request({
                    /* Here you should specify the exact link to the file you are trying to download */
                    uri: `https://github.com/pmd/pmd/releases/download/pmd_releases%2F${npm_package_pmd_version}/pmd-bin-${npm_package_pmd_version}.zip`
                })
                    .pipe(file)
                    .on("finish", () => {
                    resolve();
                })
                    .on("error", error => {
                    reject(error);
                });
            }).catch(error => {
                sfpowerkit_1.SFPowerkit.log(`Unable to download: ${error}`, sfpowerkit_1.LoggerLevel.ERROR);
            });
        }
        parseXmlReport(xmlReport, moduleName) {
            let fileCount = 0;
            let violationCount = 0;
            let reportContent = fs.readFileSync(xmlReport, "utf-8");
            xml2js.parseString(reportContent, (err, data) => {
                // If the file is not XML, or is not from PMD, return immediately
                if (!data || !data.pmd) {
                    sfpowerkit_1.SFPowerkit.log(`Empty or unrecognized PMD xml report ${xmlReport}`, sfpowerkit_1.LoggerLevel.ERROR);
                    return null;
                }
                if (!data.pmd.file || data.pmd.file.length === 0) {
                    // No files with violations, return now that it has been marked for upload
                    sfpowerkit_1.SFPowerkit.log(`A PMD report was found for module '${moduleName}' but it contains no violations`, sfpowerkit_1.LoggerLevel.INFO);
                    return null;
                }
                data.pmd.file.forEach((file) => {
                    if (file.violation) {
                        fileCount++;
                        violationCount += file.violation.length;
                    }
                });
                console.debug(`PMD analyzation complete  for module '${moduleName}' containing ${violationCount} issues, Report available at ${this.flags.report}`);
            });
            return [violationCount, fileCount];
        }
    }
    Pmd.description = messages.getMessage("commandDescription");
    Pmd.examples = [`$ sfdx sfpowerkit:source:pmd`];
    Pmd.flagsConfig = {
        directory: command_1.flags.string({
            required: false,
            char: "d",
            description: messages.getMessage("directoryFlagDescription")
        }),
        ruleset: command_1.flags.string({
            required: false,
            char: "r",
            description: messages.getMessage("rulesetFlagDescription")
        }),
        format: command_1.flags.string({
            required: false,
            char: "f",
            default: "text",
            description: messages.getMessage("formatFlagDescription")
        }),
        report: command_1.flags.string({
            required: false,
            char: "o",
            default: "pmd-output",
            description: messages.getMessage("reportFlagDescription")
        }),
        javahome: command_1.flags.string({
            required: false,
            description: messages.getMessage("javaHomeFlagDescription")
        }),
        supressoutput: command_1.flags.boolean({
            required: false,
            default: false,
            description: messages.getMessage("supressoutputFlagDescription")
        }),
        version: command_1.flags.string({
            required: false,
            default: "6.22.0",
            description: messages.getMessage("versionFlagDescription")
        })
    };
    return Pmd;
})();
exports.default = Pmd;
//# sourceMappingURL=pmd.js.map