"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const _ = __importStar(require("lodash"));
const sfpowerkit_1 = require("../../../../sfpowerkit");
const metadataInfo_1 = require("../../../../impl/metadata/metadataInfo");
const path = __importStar(require("path"));
const profileReconcile_1 = __importDefault(require("../../../../impl/source/profiles/profileReconcile"));
const metadataFiles_1 = __importDefault(require("../../../../impl/metadata/metadataFiles"));
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "profile_reconcile");
let Reconcile = /** @class */ (() => {
    class Reconcile extends command_1.SfdxCommand {
        async run() {
            sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
            let argFolder = this.flags.folder;
            let argProfileList = this.flags.profilelist;
            if (!this.flags.sourceonly) {
                if (_.isNil(this.flags.targetorg)) {
                    throw new Error("Either set sourceonly flag or provide and org for reconcile");
                }
                else {
                    this.org = await core_1.Org.create({ aliasOrUsername: this.flags.targetorg });
                }
            }
            metadataFiles_1.default.sourceOnly = this.flags.sourceonly;
            if (!_.isNil(argFolder) && argFolder.length !== 0) {
                sfpowerkit_1.SFPowerkit.setDefaultFolder(argFolder[0]);
            }
            var profileUtils = new profileReconcile_1.default(this.org, this.flags.loglevel == "debug");
            let result = [];
            let retryCount = 0;
            let success = false;
            while (!success && retryCount < 2) {
                try {
                    var reconcileProfiles = await profileUtils.reconcile(argFolder, argProfileList || [], this.flags.destfolder);
                    // Return an object to be displayed with --json
                    reconcileProfiles.forEach(file => {
                        result.push({
                            state: "Cleaned",
                            fullName: path.basename(file, metadataInfo_1.METADATA_INFO.Profile.sourceExtension),
                            type: "Profile",
                            path: path.relative(process.cwd(), file)
                        });
                    });
                    success = true;
                }
                catch (err) {
                    sfpowerkit_1.SFPowerkit.log(err, sfpowerkit_1.LoggerLevel.ERROR);
                    retryCount++;
                    if (retryCount < 2) {
                        sfpowerkit_1.SFPowerkit.log("An error occured during profile reconcile. Retry in 10 seconds", sfpowerkit_1.LoggerLevel.INFO);
                        //Wait 5 seconds
                        await this.sleep(10000);
                    }
                    else {
                        sfpowerkit_1.SFPowerkit.log("An error occured during profile reconcile. You can rerun the command after a moment.", sfpowerkit_1.LoggerLevel.ERROR);
                    }
                }
            }
            return result;
        }
        async sleep(ms) {
            return new Promise(resolve => {
                setTimeout(resolve, ms);
            });
        }
    }
    Reconcile.description = messages.getMessage("commandDescription");
    Reconcile.examples = [
        `$ sfdx sfpowerkit:source:profile:reconcile  --folder force-app -d destfolder -s`,
        `$ sfdx sfpowerkit:source:profile:reconcile  --folder force-app,module2,module3 -u sandbox -d destfolder`,
        `$ sfdx sfpowerkit:source:profile:reconcile  -u myscratchorg -d destfolder`
    ];
    //public static args = [{name: 'file'}];
    Reconcile.flagsConfig = {
        // flag with a value (-n, --name=VALUE)
        folder: command_1.flags.array({
            char: "f",
            description: messages.getMessage("folderFlagDescription"),
            required: false,
            map: (f) => f.trim()
        }),
        profilelist: command_1.flags.array({
            char: "n",
            description: messages.getMessage("nameFlagDescription"),
            required: false,
            map: (n) => n.trim()
        }),
        destfolder: command_1.flags.directory({
            char: "d",
            description: messages.getMessage("destFolderFlagDescription"),
            required: false
        }),
        sourceonly: command_1.flags.boolean({
            char: "s",
            description: messages.getMessage("sourceonlyFlagDescription"),
            required: false
        }),
        targetorg: command_1.flags.string({
            char: "u",
            description: messages.getMessage("targetorgFlagDescription"),
            required: false
        }),
        loglevel: command_1.flags.enum({
            description: "logging level for this command invocation",
            default: "info",
            required: false,
            options: [
                "trace",
                "debug",
                "info",
                "warn",
                "error",
                "fatal",
                "TRACE",
                "DEBUG",
                "INFO",
                "WARN",
                "ERROR",
                "FATAL"
            ]
        })
    };
    // Comment this out if your command does not require an org username
    Reconcile.requiresUsername = false;
    // Comment this out if your command does not support a hub org username
    //protected static supportsDevhubUsername = true;
    // Set this to true if your command requires a project workspace; 'requiresProject' is false by default
    Reconcile.requiresProject = true;
    Reconcile.result = {
        tableColumnData: {
            columns: [
                { key: "state", label: "State" },
                { key: "fullName", label: "Full Name" },
                { key: "type", label: "Type" },
                { key: "path", label: "Path" }
            ]
        },
        display() {
            if (Array.isArray(this.data) && this.data.length) {
                this.ux.table(this.data, this.tableColumnData);
            }
        }
    };
    return Reconcile;
})();
exports.default = Reconcile;
//# sourceMappingURL=reconcile.js.map