"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sfpowerkit_1 = require("../../sfpowerkit");
const queryExecutor_1 = __importDefault(require("../../utils/queryExecutor"));
const chunkArray_1 = require("../../utils/chunkArray");
const progressBar_1 = require("../../ui/progressBar");
class DependencyImpl {
    static async getDependencyMapById(conn, refMetadata) {
        let progressBar = new progressBar_1.ProgressBar().create(`Fetching dependency details `, ` metadata components`, sfpowerkit_1.LoggerLevel.INFO);
        progressBar.start(refMetadata.length);
        let dependencyMap = new Map();
        let dependencyDetailsMap = new Map();
        let filterOn = " RefMetadataComponentId ";
        for (let chunkrefMetadata of chunkArray_1.chunkArray(500, refMetadata)) {
            const results = await this.fetchDependencies(conn, filterOn, chunkrefMetadata, dependencyMap, dependencyDetailsMap);
            if (results) {
                dependencyMap = results.dependencyMap;
                dependencyDetailsMap = results.dependencyDetailsMap;
            }
            progressBar.increment(chunkrefMetadata.length);
        }
        progressBar.stop();
        return {
            dependencyMap: dependencyMap,
            dependencyDetailsMap: dependencyDetailsMap
        };
    }
    static async getDependencyMapByType(conn, refMetadata) {
        let progressBar = new progressBar_1.ProgressBar().create(`Fetching dependency details `, ` metadata components`, sfpowerkit_1.LoggerLevel.INFO);
        progressBar.start(refMetadata.length);
        let dependencyMap = new Map();
        let dependencyDetailsMap = new Map();
        let filterOn = " RefMetadataComponentType ";
        if (refMetadata.length > 500) {
            for (let chunkrefMetadata of chunkArray_1.chunkArray(500, refMetadata)) {
                const results = await this.fetchDependencies(conn, filterOn, chunkrefMetadata, dependencyMap, dependencyDetailsMap);
                if (results) {
                    dependencyMap = results.dependencyMap;
                    dependencyDetailsMap = results.dependencyDetailsMap;
                }
                progressBar.increment(chunkrefMetadata.length);
            }
        }
        else {
            const results = await this.fetchDependencies(conn, filterOn, refMetadata, dependencyMap, dependencyDetailsMap);
            if (results) {
                dependencyMap = results.dependencyMap;
                dependencyDetailsMap = results.dependencyDetailsMap;
            }
            progressBar.increment(refMetadata.length);
        }
        progressBar.stop();
        return {
            dependencyMap: dependencyMap,
            dependencyDetailsMap: dependencyDetailsMap
        };
    }
    static async fetchDependencies(conn, filterOn, refMetadata, dependencyMap, dependencyDetailsMap) {
        let query = `SELECT MetadataComponentId, MetadataComponentNamespace, MetadataComponentName, MetadataComponentType, RefMetadataComponentId, RefMetadataComponentNamespace, ` +
            `RefMetadataComponentName, RefMetadataComponentType FROM MetadataComponentDependency where ${filterOn} IN ('` +
            refMetadata.join(`','`) +
            `') `;
        let queryUtil = new queryExecutor_1.default(conn);
        let result = await queryUtil.executeQuery(query, true);
        let memberList = [];
        result.forEach(element => {
            memberList = dependencyMap.get(element.RefMetadataComponentId) || [];
            memberList.push(element.MetadataComponentId);
            dependencyMap.set(element.RefMetadataComponentId, memberList);
            dependencyDetailsMap.set(element.MetadataComponentId, {
                id: element.MetadataComponentId,
                fullName: element.MetadataComponentName,
                type: element.MetadataComponentType
            });
            dependencyDetailsMap.set(element.RefMetadataComponentId, {
                id: element.RefMetadataComponentId,
                fullName: element.RefMetadataComponentName,
                type: element.RefMetadataComponentType
            });
        });
        return {
            dependencyMap: dependencyMap,
            dependencyDetailsMap: dependencyDetailsMap
        };
    }
    static async getMemberVsPackageMap(conn) {
        let query = `SELECT CurrentPackageVersionId, MaxPackageVersionId, MinPackageVersionId, SubjectId, SubjectKeyPrefix, SubjectManageableState, SubscriberPackageId ` +
            `FROM Package2Member  WHERE (SubjectManageableState = 'installed' OR SubjectManageableState = 'installedEditable') ORDER BY SubjectId `;
        let queryUtil = new queryExecutor_1.default(conn);
        let result = await queryUtil.executeQuery(query, true);
        let packageMember = new Map();
        if (result) {
            result.forEach(cmp => {
                packageMember.set(cmp.SubjectId, cmp.SubscriberPackageId);
            });
        }
        return packageMember;
    }
    static async getPackageVsMemberMap(conn) {
        let query = `SELECT CurrentPackageVersionId, MaxPackageVersionId, MinPackageVersionId, SubjectId, SubjectKeyPrefix, SubjectManageableState, SubscriberPackageId ` +
            `FROM Package2Member  WHERE (SubjectManageableState = 'installed' OR SubjectManageableState = 'installedEditable') ORDER BY SubjectId `;
        let queryUtil = new queryExecutor_1.default(conn);
        let result = await queryUtil.executeQuery(query, true);
        let packageMember = new Map();
        let memberList = [];
        if (result) {
            result.forEach(cmp => {
                memberList = packageMember.get(cmp.SubscriberPackageId) || [];
                memberList.push(cmp.SubjectId);
                packageMember.set(cmp.SubscriberPackageId, memberList);
            });
        }
        return packageMember;
    }
    static async getMemberFromPackage(conn, packageId) {
        let query = `SELECT CurrentPackageVersionId, MaxPackageVersionId, MinPackageVersionId, SubjectId, SubjectKeyPrefix, SubjectManageableState, SubscriberPackageId ` +
            `FROM Package2Member WHERE SubscriberPackageId = '${packageId}' AND (SubjectManageableState = 'installed' OR SubjectManageableState = 'installedEditable') ORDER BY SubjectId `;
        let queryUtil = new queryExecutor_1.default(conn);
        let result = await queryUtil.executeQuery(query, true);
        let packageMember = [];
        if (result) {
            result.forEach(cmp => {
                packageMember.push(cmp.SubjectId);
            });
        }
        return packageMember;
    }
}
exports.default = DependencyImpl;
//# sourceMappingURL=dependencyImpl.js.map