"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PROFILE_PERMISSIONSET_EXTENSION = exports.UNSPLITED_METADATA = exports.METADATA_INFO = exports.MetadataInfo = exports.SOURCE_EXTENSION_REGEX = void 0;
const _ = __importStar(require("lodash"));
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
exports.SOURCE_EXTENSION_REGEX = /\.[a-zA-Z]+-meta\.xml/;
const SPLITED_TYPES = {
    CustomField: {
        suffix: "field",
        folder: "fields"
    },
    BusinessProcess: {
        suffix: "businessProcess",
        folder: "businessProcesses"
    },
    CompactLayout: {
        suffix: "compactLayout",
        folder: "compactLayouts"
    },
    FieldSet: {
        suffix: "fieldSet",
        folder: "fieldSets"
    },
    RecordType: {
        suffix: "recordType",
        folder: "recordTypes"
    },
    ListView: {
        suffix: "listView",
        folder: "listViews"
    },
    SharingReason: {
        suffix: "sharingReason",
        folder: "sharingReasons"
    },
    ValidationRule: {
        suffix: "validationRule",
        folder: "validationRules"
    },
    WebLink: {
        suffix: "webLink",
        folder: "webLinks"
    }
};
class MetadataInfo {
    static loadMetadataInfo() {
        let metadataInfo = {};
        let resourcePath = path.join(__dirname, "..", "..", "..", "resources", "metadatainfo.json");
        const fileData = fs.readFileSync(resourcePath, "utf8");
        let metadataInfoJSON = JSON.parse(fileData);
        metadataInfoJSON.metadataObjects.forEach(metadata => {
            let metadataDescribe = metadata;
            if (_.isNil(metadata.suffix)) {
                if (metadata.xmlName === "AuraDefinitionBundle") {
                    metadata.suffix = "cmp";
                    metadataDescribe.suffix = "cmp";
                }
                else if (metadata.xmlName == "LightningComponentBundle") {
                    metadata.suffix = "js";
                    metadataDescribe.suffix = "js";
                }
            }
            metadataDescribe.sourceExtension = `.${metadata.suffix}-meta.xml`;
            if (metadata.inFolder) {
                let folderExtensionPrefix = metadata.suffix;
                if (_.isNil(metadata.suffix)) {
                    folderExtensionPrefix =
                        metadata.xmlName.charAt(0).toLowerCase + metadata.xmlName.slice(1);
                }
                metadataDescribe.folderExtension = `.${folderExtensionPrefix}Folder-meta.xml`;
            }
            //Generate Describe of cheildItems if exists
            if (!_.isNil(metadata.childXmlNames)) {
                metadata.childXmlNames.forEach(element => {
                    let splitedElement = SPLITED_TYPES[element];
                    if (!_.isNil(splitedElement)) {
                        let childDescribe = {};
                        childDescribe.directoryName = SPLITED_TYPES[element].folder;
                        childDescribe.suffix = SPLITED_TYPES[element].suffix;
                        childDescribe.xmlName = element;
                        childDescribe.inFolder = false;
                        childDescribe.metaFile = false;
                        childDescribe.isChildComponent = true;
                        childDescribe.sourceExtension = `.${SPLITED_TYPES[element].suffix}-meta.xml`;
                        metadataInfo[childDescribe.xmlName] = childDescribe;
                    }
                });
            }
            metadataInfo[metadataDescribe.xmlName] = metadataDescribe;
        });
        return metadataInfo;
    }
    static getMetadataName(metadataFile, validateSourceExtension = true) {
        let matcher = metadataFile.match(exports.SOURCE_EXTENSION_REGEX);
        let extension = "";
        if (matcher) {
            extension = matcher[0];
        }
        else {
            extension = path.parse(metadataFile).ext;
        }
        //SfPowerKit.ux.log(extension);
        let metadataName = "";
        const auraRegExp = new RegExp("aura");
        const lwcRegExp = new RegExp("lwc");
        const staticResourceRegExp = new RegExp("staticresources");
        const documentRegExp = new RegExp("documents");
        if (auraRegExp.test(metadataFile) &&
            (exports.SOURCE_EXTENSION_REGEX.test(metadataFile) || !validateSourceExtension)) {
            metadataName = exports.METADATA_INFO.AuraDefinitionBundle.xmlName;
        }
        else if (lwcRegExp.test(metadataFile) &&
            (exports.SOURCE_EXTENSION_REGEX.test(metadataFile) || !validateSourceExtension)) {
            metadataName = exports.METADATA_INFO.LightningComponentBundle.xmlName;
        }
        else if (staticResourceRegExp.test(metadataFile) &&
            (exports.SOURCE_EXTENSION_REGEX.test(metadataFile) || !validateSourceExtension)) {
            metadataName = exports.METADATA_INFO.StaticResource.xmlName;
        }
        else if (documentRegExp.test(metadataFile) &&
            (exports.SOURCE_EXTENSION_REGEX.test(metadataFile) || !validateSourceExtension)) {
            metadataName = exports.METADATA_INFO.Document.xmlName;
        }
        else {
            let keys = Object.keys(exports.METADATA_INFO);
            for (let i = 0; i < keys.length; i++) {
                let metaDescribe = exports.METADATA_INFO[keys[i]];
                if (metaDescribe.sourceExtension === extension ||
                    ("." + metaDescribe.suffix === extension &&
                        !validateSourceExtension) ||
                    metaDescribe.folderExtension === extension) {
                    metadataName = metaDescribe.xmlName;
                    break;
                }
            }
        }
        return metadataName;
    }
}
exports.MetadataInfo = MetadataInfo;
exports.METADATA_INFO = MetadataInfo.loadMetadataInfo();
exports.UNSPLITED_METADATA = [
    exports.METADATA_INFO.Workflow,
    exports.METADATA_INFO.SharingRules,
    exports.METADATA_INFO.CustomLabels,
    exports.METADATA_INFO.Profile,
    exports.METADATA_INFO.PermissionSet
];
exports.PROFILE_PERMISSIONSET_EXTENSION = [
    exports.METADATA_INFO.Profile,
    exports.METADATA_INFO.PermissionSet
];
//# sourceMappingURL=metadataInfo.js.map