"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const getDefaults_1 = __importDefault(require("../../../utils/getDefaults"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const chunkArray_1 = require("../../../utils/chunkArray");
const progressBar_1 = require("../../../ui/progressBar");
const getDefaults_2 = __importDefault(require("../../../utils/getDefaults"));
const async_retry_1 = __importDefault(require("async-retry"));
let MetadataSummaryInfoFetcher = /** @class */ (() => {
    class MetadataSummaryInfoFetcher {
        static async fetchMetadataSummaryFromAnOrg(conn, isDisplayProgressBar = false, filterTypes = MetadataSummaryInfoFetcher.NotSupportedTypes) {
            let metadataMap = new Map();
            let types = [];
            let result = await conn.metadata.describe(getDefaults_1.default.getApiVersion());
            result.metadataObjects.forEach(metadata => {
                //Not supported .. ignore
                if (!this.NotSupportedTypes.includes(metadata.xmlName)) {
                    types.push({ type: metadata.xmlName });
                }
                //Has childs.. check for each child and add to the list
                if (metadata.childXmlNames) {
                    for (let childMetadata of metadata.childXmlNames) {
                        if (!this.NotSupportedTypes.includes(childMetadata)) {
                            types.push({ type: childMetadata });
                        }
                    }
                }
            });
            let progressBar = new progressBar_1.ProgressBar().create(`Fetching  Metadata  Types From the Org `, ` metdata types`, sfpowerkit_1.LoggerLevel.INFO);
            progressBar.start(types.length);
            //Fetch Summary Info in chunks of three
            for (let typesInChunk of chunkArray_1.chunkArray(3, types)) {
                try {
                    metadataMap = await this.fetchMetadataSummaryByTypesFromAnOrg(conn, typesInChunk, metadataMap);
                    progressBar.increment(typesInChunk.length);
                }
                catch (error) {
                    throw new core_1.SfdxError("Unable to retrieve metadata from the org");
                }
            }
            progressBar.stop();
            return metadataMap;
        }
        static async fetchMetadataSummaryByTypesFromAnOrg(conn, types, metadataMap) {
            return await async_retry_1.default(async (bail) => {
                try {
                    let results = await conn.metadata.list(types, getDefaults_2.default.getApiVersion());
                    for (let result of results) {
                        metadataMap.set(result.id, {
                            id: result.id,
                            fullName: result.fullName,
                            type: result.type
                        });
                    }
                    return metadataMap;
                }
                catch (error) {
                    throw error;
                }
            }, { retries: 3, minTimeout: 2000 });
        }
    }
    MetadataSummaryInfoFetcher.NotSupportedTypes = [
        "AccountForecastSettings",
        "Icon",
        "GlobalValueSet",
        "StandardValueSet",
        "CustomPermission",
        "EscalationRules",
        "RecordActionDeployment",
        "EscalationRule",
        "ApprovalProcess",
        "SiteDotCom",
        "BrandingSet",
        "NetworkBranding",
        "AuthProvider",
        "ContentAsset",
        "CustomSite",
        "EmbeddedServiceConfig",
        "UIObjectRelationConfig",
        "CareProviderSearchConfig",
        "EmbeddedServiceBranding",
        "EmbeddedServiceFlowConfig",
        "EmbeddedServiceMenuSettings",
        "SalesAgreementSettings",
        "ActionLinkGroupTemplate",
        "TransactionSecurityPolicy",
        "SynonymDictionary",
        "RecommendationStrategy",
        "UserCriteria",
        "ModerationRule",
        "CMSConnectSource",
        "FlowCategory",
        "Settings",
        "PlatformCachePartition",
        "LightningBolt",
        "LightningExperienceTheme",
        "LightningOnboardingConfig",
        "CorsWhitelistOrigin",
        "CustomHelpMenuSection",
        "Prompt",
        "Report",
        "Dashboard",
        "AnalyticSnapshot",
        "Role",
        "Group",
        "Community",
        "ChatterExtension",
        "PlatformEventChannel",
        "CommunityThemeDefinition",
        "CommunityTemplateDefinition",
        "NavigationMenu",
        "ManagedTopics",
        "ManagedTopic",
        "KeywordList",
        "InstalledPackage",
        "Scontrol",
        "Certificate",
        "LightningMessageChannel",
        "CaseSubjectParticle",
        "ExternalDataSource",
        "ExternalServiceRegistration",
        "Index",
        "CustomFeedFilter",
        "PostTemplate",
        "ProfilePasswordPolicy",
        "ProfileSessionSetting",
        "MyDomainDiscoverableLogin",
        "OauthCustomScope",
        "DataCategoryGroup",
        "RemoteSiteSetting",
        "CspTrustedSite",
        "RedirectWhitelistUrl",
        "CleanDataService",
        "Skill",
        "ServiceChannel",
        "QueueRoutingConfig",
        "ServicePresenceStatus",
        "PresenceDeclineReason",
        "PresenceUserConfig",
        "EclairGeoData",
        "ChannelLayout",
        "CallCenter",
        "TimeSheetTemplate",
        "CanvasMetadata",
        "MobileApplicationDetail",
        "CustomNotificationType",
        "NotificationTypeConfig",
        "DelegateGroup",
        "ManagedContentType",
        "EmailServicesFunction",
        "SamlSsoConfig"
    ];
    return MetadataSummaryInfoFetcher;
})();
exports.default = MetadataSummaryInfoFetcher;
//# sourceMappingURL=metadataSummaryInfoFetcher.js.map