"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const rimraf = __importStar(require("rimraf"));
const xml2js = __importStar(require("xml2js"));
const util = __importStar(require("util"));
// tslint:disable-next-line:ordered-imports
var path = require("path");
const checkRetrievalStatus_1 = require("../../utils/checkRetrievalStatus");
const checkDeploymentStatus_1 = require("../../utils/checkDeploymentStatus");
const extract_1 = require("../../utils/extract");
const zipDirectory_1 = require("../../utils/zipDirectory");
const sfpowerkit_1 = require("../../sfpowerkit");
class RelaxIPRangeImpl {
    static async setIp(conn, username, ipRangeToSet) {
        const apiversion = await conn.retrieveMaxApiVersion();
        let retrieveRequest = {
            apiVersion: apiversion
        };
        //Retrieve Duplicate Rule
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: { name: "Settings", members: "Security" }
        };
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        sfpowerkit_1.SFPowerkit.log(`Fetching Ip range from ${conn.getUsername()}`, sfpowerkit_1.LoggerLevel.DEBUG);
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId);
        if (!metadata_retrieve_result.zipFile)
            sfpowerkit_1.SFPowerkit.log("Unable to find the settings", sfpowerkit_1.LoggerLevel.ERROR);
        let retriveLocation = `temp_sfpowerkit_${retrievedId}`;
        //Extract Matching Rule
        var zipFileName = `${retriveLocation}/unpackaged.zip`;
        fs.mkdirSync(retriveLocation);
        fs.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64"
        });
        await extract_1.extract(`./${retriveLocation}/unpackaged.zip`, retriveLocation);
        fs.unlinkSync(zipFileName);
        let resultFile = `${retriveLocation}/settings/Security.settings`;
        if (fs.existsSync(path.resolve(resultFile))) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            let retrieve_securitySetting = await parseString(fs.readFileSync(path.resolve(resultFile)));
            sfpowerkit_1.SFPowerkit.log(`Ip range to set :` + JSON.stringify(ipRangeToSet), sfpowerkit_1.LoggerLevel.INFO);
            if (!retrieve_securitySetting.SecuritySettings.networkAccess) {
                retrieve_securitySetting.SecuritySettings.networkAccess = {
                    ipRanges: ipRangeToSet
                };
                sfpowerkit_1.SFPowerkit.log(`Currently No Ip range set in ${conn.getUsername()}`, sfpowerkit_1.LoggerLevel.DEBUG);
            }
            else {
                let currentRange = retrieve_securitySetting.SecuritySettings.networkAccess.ipRanges;
                sfpowerkit_1.SFPowerkit.log(`Org ${conn.getUsername()} has current range : ` +
                    JSON.stringify(currentRange), sfpowerkit_1.LoggerLevel.DEBUG);
                if (currentRange.constructor === Array) {
                    ipRangeToSet.concat(currentRange);
                }
                else {
                    ipRangeToSet.push(currentRange);
                }
                retrieve_securitySetting.SecuritySettings.networkAccess.ipRanges = ipRangeToSet;
            }
            let builder = new xml2js.Builder();
            var xml = builder.buildObject(retrieve_securitySetting);
            fs.writeFileSync(resultFile, xml);
            var zipFile = `${retriveLocation}/package.zip`;
            await zipDirectory_1.zipDirectory(retriveLocation, zipFile);
            //Deploy Trigger
            conn.metadata.pollTimeout = 300;
            let deployId;
            var zipStream = fs.createReadStream(zipFile);
            await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                if (error) {
                    return console.error(error);
                }
                deployId = result;
            });
            sfpowerkit_1.SFPowerkit.log(`Setting Ip range with ID  ${deployId.id} to ${conn.getUsername()}`, sfpowerkit_1.LoggerLevel.DEBUG);
            let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
            rimraf.sync(retriveLocation);
            if (!metadata_deploy_result.success) {
                sfpowerkit_1.SFPowerkit.log(`Unable to set ip range : ${metadata_deploy_result.details["componentFailures"]["problem"]}`, sfpowerkit_1.LoggerLevel.ERROR);
                return { username: username, success: false };
            }
            else {
                sfpowerkit_1.SFPowerkit.log(`Ip range is successfully set in ${conn.getUsername()}`, sfpowerkit_1.LoggerLevel.INFO);
                return { username: username, success: true };
            }
        }
    }
}
exports.default = RelaxIPRangeImpl;
//# sourceMappingURL=relaxIPRangeImpl.js.map