"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
class PoolDeleteImpl {
    constructor(hubOrg, apiversion, tag, mypool, allScratchOrgs) {
        this.hubOrg = hubOrg;
        this.apiversion = apiversion;
        this.tag = tag;
        this.mypool = mypool;
        this.allScratchOrgs = allScratchOrgs;
    }
    async execute() {
        const results = (await scratchOrgUtils_1.default.getScratchOrgsByTag(this.tag, this.hubOrg, this.mypool, !this.allScratchOrgs));
        let scratchOrgToDelete = new Array();
        if (results.records.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`${this.tag} pool has ${results.records.length} Scratch orgs available`, core_1.LoggerLevel.TRACE);
            let scrathOrgIds = [];
            for (let element of results.records) {
                let soDetail = {};
                soDetail.orgId = element.ScratchOrg;
                soDetail.loginURL = element.LoginUrl;
                soDetail.username = element.SignupUsername;
                soDetail.expityDate = element.ExpirationDate;
                soDetail.status = "Deleted";
                scratchOrgToDelete.push(soDetail);
                scrathOrgIds.push(`'${element.Id}'`);
            }
            let activeScrathOrgs = await scratchOrgUtils_1.default.getActiveScratchOrgsByInfoId(this.hubOrg, scrathOrgIds.join(","));
            if (activeScrathOrgs.records.length > 0) {
                for (let ScratchOrg of activeScrathOrgs.records) {
                    await scratchOrgUtils_1.default.deleteScratchOrg(this.hubOrg, this.apiversion, ScratchOrg.Id);
                    sfpowerkit_1.SFPowerkit.log(`Scratch org with username ${ScratchOrg.SignupUsername} is deleted successfully`, core_1.LoggerLevel.TRACE);
                }
            }
        }
        return scratchOrgToDelete;
    }
}
exports.default = PoolDeleteImpl;
//# sourceMappingURL=PoolDeleteImpl.js.map