"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const fs = __importStar(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const bottleneck_1 = __importDefault(require("bottleneck"));
const util_1 = require("util");
const relaxIPRangeImpl_1 = __importDefault(require("../../org/relaxIPRangeImpl"));
const limiter = new bottleneck_1.default({
    maxConcurrent: 10
});
class PoolCreateImpl {
    constructor(poolconfigFilePath, hubOrg, apiversion) {
        this.totalAllocated = 0;
        this.scriptExecutorWrappedForBottleneck = limiter.wrap(this.scriptExecutor);
        this.ipRangeRelaxerWrappedForBottleneck = limiter.wrap(this.ipRangeRelaxer);
        this.arrayToObject = (array, keyfield) => array.reduce((obj, item) => {
            obj[item[keyfield]] = item;
            return obj;
        }, {});
        this.poolconfigFilePath = poolconfigFilePath;
        this.hubOrg = hubOrg;
        this.apiversion = apiversion;
    }
    async poolScratchOrgs() {
        let scriptExecPromises = new Array();
        let ipRangeExecPromises = new Array();
        await this.hubOrg.refreshAuth();
        this.hubConn = this.hubOrg.getConnection();
        let preRequisiteCheck = await scratchOrgUtils_1.default.checkForPreRequisite(this.hubOrg);
        if (!preRequisiteCheck) {
            sfpowerkit_1.SFPowerkit.log("Required Prerequisite fields are missing in the DevHub, Please look into the wiki to getting the fields deployed in DevHub", core_1.LoggerLevel.ERROR);
            return false;
        }
        //Read pool config file
        if (!fs.existsSync(this.poolconfigFilePath)) {
            sfpowerkit_1.SFPowerkit.log("Poll Config Path not provided, Unable to create pool without this file", core_1.LoggerLevel.ERROR);
            return false;
        }
        this.poolConfig = JSON.parse(fs.readFileSync(this.poolconfigFilePath).toString());
        //Validate Inputs
        if (util_1.isNullOrUndefined(this.poolConfig.pool.config_file_path)) {
            sfpowerkit_1.SFPowerkit.log("Scratch Org Config Path not provided, Unable to create pool without this file", core_1.LoggerLevel.ERROR);
            return true;
        }
        if (util_1.isNullOrUndefined(this.poolConfig.pool.expiry) ||
            util_1.isNullOrUndefined(this.poolConfig.pool.tag)) {
            sfpowerkit_1.SFPowerkit.log("Some Key parameters are missing in the schema,Please consult the documentation", core_1.LoggerLevel.ERROR);
            return true;
        }
        this.validateScriptFile();
        if (this.poolConfig.poolUsers && this.poolConfig.poolUsers.length > 0)
            this.poolConfig.pool.user_mode = true;
        else
            this.poolConfig.pool.user_mode = false;
        //Set Tag Only mode activated for the default use case
        if (this.poolConfig.pool.user_mode == false)
            this.setASingleUserForTagOnlyMode();
        sfpowerkit_1.SFPowerkit.log("Pool Config:" + JSON.stringify(this.poolConfig), core_1.LoggerLevel.TRACE);
        if (util_1.isNullOrUndefined(this.poolConfig.pool.relax_ip_ranges) &&
            !this.poolConfig.pool.user_mode) {
            sfpowerkit_1.SFPowerkit.log("IP Ranges are not relaxed, The created scratch org's will have the pool creators email as Admin Email and has to be verifed before use", core_1.LoggerLevel.WARN);
        }
        //fetch current status limits
        await this.fetchCurrentLimits();
        //Compute allocation
        this.totalToBeAllocated = await this.computeAllocation();
        if (this.totalToBeAllocated === 0) {
            if (this.limits.ActiveScratchOrgs.Remaining > 0)
                sfpowerkit_1.SFPowerkit.log(`The tag provided ${this.poolConfig.pool.tag} is currently at the maximum capacity , No scratch orgs will be allocated`, core_1.LoggerLevel.INFO);
            else
                sfpowerkit_1.SFPowerkit.log(`There is no capacity to create a pool at this time, Please try again later`, core_1.LoggerLevel.INFO);
            return;
        }
        //Generate Scratch Orgs
        await this.generateScratchOrgs();
        // Assign workers to executed scripts
        let ts = Math.floor(Date.now() / 1000);
        for (let poolUser of this.poolConfig.poolUsers) {
            poolUser.scratchOrgs.forEach(scratchOrg => {
                sfpowerkit_1.SFPowerkit.log(JSON.stringify(scratchOrg), core_1.LoggerLevel.DEBUG);
                if (this.poolConfig.pool.relax_ip_ranges) {
                    let resultForIPRelaxation = this.ipRangeRelaxerWrappedForBottleneck(scratchOrg);
                    ipRangeExecPromises.push(resultForIPRelaxation);
                }
                if (this.scriptFileExists) {
                    let result = this.scriptExecutorWrappedForBottleneck(this.poolConfig.pool.script_file_path, scratchOrg, this.hubOrg.getUsername());
                    scriptExecPromises.push(result);
                }
            });
        }
        //Wait for scripts to finish execution
        if (this.poolConfig.pool.relax_ip_ranges)
            this.ipRangeExecResults = await Promise.all(ipRangeExecPromises);
        let scriptExecResults = await Promise.all(scriptExecPromises);
        if (this.scriptFileExists) {
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(scriptExecResults), core_1.LoggerLevel.TRACE);
            ts = Math.floor(Date.now() / 1000) - ts;
            sfpowerkit_1.SFPowerkit.log(`Script Execution completed in ${ts} Seconds`, core_1.LoggerLevel.INFO);
        }
        //Commit Succesfull Scratch Orgs
        let commit_result = await this.commitGeneratedScratchOrgs();
        if (this.totalAllocated > 0) {
            sfpowerkit_1.SFPowerkit.log(`Request for provisioning ${this.totalToBeAllocated} scratchOrgs of which ${this.totalAllocated} were allocated with ${commit_result.success} success and ${commit_result.failed} failures`, core_1.LoggerLevel.INFO);
        }
        else {
            sfpowerkit_1.SFPowerkit.log(`Request for provisioning ${this.totalToBeAllocated} scratchOrgs of which ${this.totalAllocated} were allocated with ${commit_result.success} success and ${commit_result.failed} failures`, core_1.LoggerLevel.INFO);
        }
        return true;
    }
    validateScriptFile() {
        if (util_1.isNullOrUndefined(this.poolConfig.pool.script_file_path)) {
            sfpowerkit_1.SFPowerkit.log("Script Path not provided, will crete a pool of scratch orgs without any post creation steps", core_1.LoggerLevel.WARN);
            this.scriptFileExists = false;
        }
        else if (fs.existsSync(this.poolConfig.pool.script_file_path)) {
            this.scriptFileExists = true;
        }
        else {
            sfpowerkit_1.SFPowerkit.log("Unable to locate Script File path, will crete a pool of scratch orgs without any post creation steps", core_1.LoggerLevel.WARN);
            this.scriptFileExists = false;
        }
    }
    setASingleUserForTagOnlyMode() {
        //Remove any existing pool Config for pool users
        if (this.poolConfig.poolUsers)
            delete this.poolConfig.poolUsers;
        let poolUser = {
            min_allocation: this.poolConfig.pool.max_allocation,
            max_allocation: this.poolConfig.pool.max_allocation,
            is_build_pooluser: false,
            expiry: this.poolConfig.pool.expiry,
            priority: 1
        };
        //Add a single user
        this.poolConfig.poolUsers = [];
        this.poolConfig.poolUsers.push(poolUser);
        this.poolConfig.pool.user_mode = false;
    }
    async fetchCurrentLimits() {
        try {
            this.limits = await scratchOrgUtils_1.default.getScratchOrgLimits(this.hubOrg, this.apiversion);
        }
        catch (error) {
            sfpowerkit_1.SFPowerkit.log("Unable to connect to DevHub", core_1.LoggerLevel.ERROR);
            return;
        }
        sfpowerkit_1.SFPowerkit.log(`Active Scratch Orgs Remaining: ${this.limits.ActiveScratchOrgs.Remaining} out of ${this.limits.ActiveScratchOrgs.Max}`, core_1.LoggerLevel.TRACE);
    }
    async computeAllocation() {
        //Compute current pool requirement
        if (this.poolConfig.pool.user_mode) {
            //Retrieve Number of active SOs in the org
            let scratchOrgsResult = await scratchOrgUtils_1.default.getScratchOrgsByTag(this.poolConfig.pool.tag, this.hubOrg, false, false);
            scratchOrgsResult.records = scratchOrgsResult.records.sort();
            let scratchOrgsRecordAsMapByUser = scratchOrgsResult.records.reduce(function (obj, v) {
                obj[v.SignupEmail] = (obj[v.SignupEmail] || 0) + 1;
                return obj;
            }, {});
            sfpowerkit_1.SFPowerkit.log(JSON.stringify(scratchOrgsRecordAsMapByUser), core_1.LoggerLevel.TRACE);
            return this.allocateScratchOrgsPerUser(this.limits.ActiveScratchOrgs.Remaining, scratchOrgsRecordAsMapByUser, this.poolConfig.poolUsers);
        }
        else {
            let activeCount = await scratchOrgUtils_1.default.getCountOfActiveScratchOrgsByTag(this.poolConfig.pool.tag, this.hubOrg);
            return this.allocateScratchOrgsPerTag(this.limits.ActiveScratchOrgs.Remaining, activeCount, this.poolConfig.pool.tag, this.poolConfig.poolUsers[0]);
        }
    }
    async generateScratchOrgs() {
        //Generate Scratch Orgs
        for (let poolUser of this.poolConfig.poolUsers) {
            let count = 1;
            poolUser.scratchOrgs = new Array();
            for (let i = 0; i < poolUser.to_allocate; i++) {
                sfpowerkit_1.SFPowerkit.log(`Creating Scratch  Org for ${count} of ${this.totalToBeAllocated}..`, core_1.LoggerLevel.INFO);
                try {
                    let scratchOrg = await scratchOrgUtils_1.default.createScratchOrg(count, poolUser.username, this.poolConfig.pool.config_file_path, poolUser.expiry ? poolUser.expiry : this.poolConfig.pool.expiry, this.hubOrg);
                    poolUser.scratchOrgs.push(scratchOrg);
                    this.totalAllocated++;
                }
                catch (error) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to provision scratch org  ${count} ..   `, core_1.LoggerLevel.INFO);
                }
                count++;
            }
        }
    }
    async commitGeneratedScratchOrgs() {
        //Store Username Passwords
        let failed = 0;
        let success = 0;
        if (!util_1.isNullOrUndefined(this.poolConfig.pool.relax_ip_ranges))
            this.ipRangeExecResultsAsObject = this.arrayToObject(this.ipRangeExecResults, "username");
        for (let poolUser of this.poolConfig.poolUsers) {
            await scratchOrgUtils_1.default.getScratchOrgRecordId(poolUser.scratchOrgs, this.hubOrg);
            for (let scratchOrg of poolUser.scratchOrgs) {
                if (this.poolConfig.pool.relax_ip_ranges &&
                    !this.ipRangeExecResultsAsObject[scratchOrg.username]["success"])
                    scratchOrg.isScriptExecuted = false;
                //Just commit
                if (!this.scriptFileExists)
                    scratchOrg.isScriptExecuted = true;
                if (scratchOrg.isScriptExecuted) {
                    try {
                        await scratchOrgUtils_1.default.setScratchOrgInfo({
                            Id: scratchOrg.recordId,
                            Pooltag__c: this.poolConfig.pool.tag,
                            Password__c: scratchOrg.password
                        }, this.hubOrg);
                        success++;
                        continue;
                    }
                    catch (error) {
                        //Failed to set passwords .. warn and continue
                        sfpowerkit_1.SFPowerkit.log(`Unable to set record for ScracthOrg ${scratchOrg.username}`, core_1.LoggerLevel.TRACE);
                        continue;
                    }
                }
                sfpowerkit_1.SFPowerkit.log(`Failed to execute scripts for ${scratchOrg.username}.. Returning to Pool`, core_1.LoggerLevel.WARN);
                try {
                    //Delete failed scratch org
                    await scratchOrgUtils_1.default.deleteScratchOrg(this.hubOrg, this.apiversion, scratchOrg.recordId);
                }
                catch (error) {
                    sfpowerkit_1.SFPowerkit.log(`Unable to delete the scratchorg ${scratchOrg.username}..`, core_1.LoggerLevel.WARN);
                }
                failed++;
            }
        }
        return { success: success, failed: failed };
    }
    allocateScratchOrgsPerTag(remainingScratchOrgs, countOfActiveScratchOrgs, tag, poolUser) {
        sfpowerkit_1.SFPowerkit.log("Remaining ScratchOrgs" + remainingScratchOrgs, core_1.LoggerLevel.TRACE);
        poolUser.current_allocation = countOfActiveScratchOrgs;
        poolUser.to_allocate = 0;
        poolUser.to_satisfy_max =
            poolUser.max_allocation - poolUser.current_allocation > 0
                ? poolUser.max_allocation - poolUser.current_allocation
                : 0;
        if (poolUser.to_satisfy_max > 0 &&
            poolUser.to_satisfy_max <= remainingScratchOrgs) {
            poolUser.to_allocate = poolUser.to_satisfy_max;
        }
        else if (poolUser.to_satisfy_max > 0 &&
            poolUser.to_satisfy_max > remainingScratchOrgs) {
            poolUser.to_allocate = remainingScratchOrgs;
        }
        sfpowerkit_1.SFPowerkit.log("Computed Allocation" + JSON.stringify(poolUser), core_1.LoggerLevel.TRACE);
        return poolUser.to_allocate;
    }
    allocateScratchOrgsPerUser(remainingScratchOrgs, scratchOrgsRecordAsMapByUser, poolUsers) {
        let totalToBeAllocated = 0;
        //sort pooleconfig.poolusers based on priority
        poolUsers = poolUsers.sort((a, b) => a.priority - b.priority);
        let totalMaxOrgRequired = 0, totalMinOrgRequired = 0;
        poolUsers.forEach(pooluser => {
            sfpowerkit_1.SFPowerkit.log(pooluser, core_1.LoggerLevel.TRACE);
            pooluser.to_allocate = 0;
            if (scratchOrgsRecordAsMapByUser[pooluser.username]) {
                pooluser.current_allocation =
                    scratchOrgsRecordAsMapByUser[pooluser.username];
                pooluser.to_satisfy_max =
                    pooluser.max_allocation - pooluser.current_allocation > 0
                        ? pooluser.max_allocation - pooluser.current_allocation
                        : 0;
                pooluser.to_satisfy_min =
                    pooluser.min_allocation - pooluser.current_allocation > 0
                        ? pooluser.min_allocation - pooluser.current_allocation
                        : 0;
            }
            else {
                pooluser.current_allocation = 0;
                pooluser.to_satisfy_max = pooluser.max_allocation;
                pooluser.to_satisfy_min = pooluser.min_allocation;
            }
            totalMaxOrgRequired += pooluser.to_satisfy_max;
            totalMinOrgRequired += pooluser.to_satisfy_min;
        });
        //All good..
        if (totalMaxOrgRequired <= remainingScratchOrgs) {
            // Satisfy max. allocate max
            poolUsers.forEach(pooluser => {
                pooluser.to_allocate = pooluser.to_satisfy_max;
                totalToBeAllocated += pooluser.to_satisfy_max;
            });
        }
        else if (totalMinOrgRequired <= remainingScratchOrgs) {
            // Satisfy min
            //First allocate minimum to everyone
            poolUsers.forEach(pooluser => {
                pooluser.to_allocate = pooluser.to_satisfy_min;
                totalToBeAllocated += pooluser.to_satisfy_min;
            });
            //Check for left overs
            let leftOver = remainingScratchOrgs - totalMinOrgRequired;
            if (leftOver > 0) {
                //Allocate LeftOver in a round robin model
                while (leftOver >= 0) {
                    poolUsers.forEach(pooluser => {
                        if (leftOver == 0)
                            return;
                        if (pooluser.current_allocation + pooluser.to_allocate <
                            pooluser.to_satisfy_max) {
                            pooluser.to_allocate++;
                            totalToBeAllocated++;
                            leftOver--;
                        }
                    });
                }
            }
        }
        else {
            let leftOver = remainingScratchOrgs;
            //Allocate LeftOver in a round robin model
            while (leftOver >= 0) {
                poolUsers.forEach(pooluser => {
                    if (pooluser.current_allocation + pooluser.to_allocate <
                        pooluser.to_satisfy_max) {
                        pooluser.to_allocate++;
                        totalToBeAllocated++;
                        leftOver--;
                    }
                });
            }
        }
        return totalToBeAllocated;
    }
    async ipRangeRelaxer(scratchOrg) {
        //executue using bash
        sfpowerkit_1.SFPowerkit.log(`Relaxing ip ranges for scratchOrg with user ${scratchOrg.username}`, core_1.LoggerLevel.INFO);
        const connection = await core_1.Connection.create({
            authInfo: await core_1.AuthInfo.create({ username: scratchOrg.username })
        });
        return relaxIPRangeImpl_1.default.setIp(connection, scratchOrg.username, this.poolConfig.pool.relax_ip_ranges);
    }
    async scriptExecutor(scriptFilePath, scratchOrg, hubOrgUserName) {
        //executue using bash
        let cmd;
        let logRestricter = 0;
        scriptFilePath = path_1.default.normalize(scriptFilePath);
        if (process.platform != "win32") {
            cmd = `bash ${scriptFilePath}  ${scratchOrg.username}  ${hubOrgUserName} `;
        }
        else {
            cmd = `cmd.exe /c ${scriptFilePath}  ${scratchOrg.username}  ${hubOrgUserName}`;
        }
        sfpowerkit_1.SFPowerkit.log(`Executing command: ${cmd}`, core_1.LoggerLevel.INFO);
        return new Promise((resolve, reject) => {
            let ls = child_process_1.exec(cmd, { cwd: process.cwd() }, (error, stdout, stderr) => {
                if (error) {
                    sfpowerkit_1.SFPowerkit.log(`failed to execute script for ${scratchOrg.username}`, core_1.LoggerLevel.WARN);
                    scratchOrg.isScriptExecuted = false;
                    resolve({
                        isSuccess: false,
                        message: error.message,
                        scratchOrgUsername: scratchOrg.username,
                        status: "failure"
                    });
                }
                if (stderr)
                    sfpowerkit_1.SFPowerkit.log(stderr, core_1.LoggerLevel.DEBUG);
                sfpowerkit_1.SFPowerkit.log(`Executing script for ${scratchOrg.alias} with username: ${scratchOrg.username}`, core_1.LoggerLevel.INFO);
                sfpowerkit_1.SFPowerkit.log(stdout, core_1.LoggerLevel.DEBUG);
                scratchOrg.isScriptExecuted = true;
                resolve({
                    isSuccess: true,
                    message: stdout,
                    scratchOrgUsername: scratchOrg.username,
                    status: "success"
                });
            });
            ls.stdout.on("data", function (data) {
                //Print only processing if there is more than 5 logs, Restrict Verbose scripts like sfpowerkit package dependencies install
                if (logRestricter % 10 == 0) {
                    sfpowerkit_1.SFPowerkit.log(`Processing for ${scratchOrg.alias} with ${scratchOrg.username}: IN_PROGRESS....`, core_1.LoggerLevel.INFO);
                }
                logRestricter++;
            });
        });
    }
}
exports.default = PoolCreateImpl;
//# sourceMappingURL=poolCreateImpl.js.map