"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
class PoolFetchImpl {
    constructor(hubOrg, apiversion, tag, mypool) {
        this.hubOrg = hubOrg;
        this.apiversion = apiversion;
        this.tag = tag;
        this.mypool = mypool;
    }
    async execute() {
        const results = (await scratchOrgUtils_1.default.getScratchOrgsByTag(this.tag, this.hubOrg, this.mypool, true));
        let soDetail;
        if (results.records.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`${this.tag} pool has ${results.records.length} Scratch orgs available`, core_1.LoggerLevel.TRACE);
            for (let element of results.records) {
                let allocateSO = await scratchOrgUtils_1.default.setScratchOrgInfo({ Id: element.Id, Allocation_status__c: "Allocate" }, this.hubOrg);
                if (allocateSO === true) {
                    sfpowerkit_1.SFPowerkit.log(`Scratch org ${element.SignupUsername} is allocated from the pool. Expiry date is ${element.ExpirationDate}`, core_1.LoggerLevel.TRACE);
                    soDetail = {};
                    soDetail.orgId = element.ScratchOrg;
                    soDetail.loginURL = element.LoginUrl;
                    soDetail.username = element.SignupUsername;
                    soDetail.password = element.Password__c;
                    soDetail.expityDate = element.ExpirationDate;
                    soDetail.status = "Assigned";
                    break;
                }
                else {
                    sfpowerkit_1.SFPowerkit.log(`Scratch org ${element.SignupUsername} allocation failed. trying to get another Scratch org from ${this.tag} pool`, core_1.LoggerLevel.TRACE);
                }
            }
        }
        if (results.records.length == 0 || !soDetail) {
            throw new core_1.SfdxError(`No scratch org available at the moment for ${this.tag}, try again in sometime.`);
        }
        return soDetail;
    }
}
exports.default = PoolFetchImpl;
//# sourceMappingURL=poolFetchImpl.js.map