"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@salesforce/core");
const sfpowerkit_1 = require("../../../sfpowerkit");
const scratchOrgUtils_1 = __importDefault(require("../../../utils/scratchOrgUtils"));
class PoolListImpl {
    constructor(hubOrg, apiversion, tag, mypool, allScratchOrgs) {
        this.hubOrg = hubOrg;
        this.apiversion = apiversion;
        this.tag = tag;
        this.mypool = mypool;
        this.allScratchOrgs = allScratchOrgs;
    }
    async execute() {
        const results = (await scratchOrgUtils_1.default.getScratchOrgsByTag(this.tag, this.hubOrg, this.mypool, !this.allScratchOrgs));
        let scratchOrgToDelete = new Array();
        if (results.records.length > 0) {
            sfpowerkit_1.SFPowerkit.log(`${this.tag} pool has ${results.records.length} Scratch orgs available`, core_1.LoggerLevel.TRACE);
            for (let element of results.records) {
                let soDetail = {};
                soDetail.tag = element.Pooltag__c;
                soDetail.orgId = element.ScratchOrg;
                soDetail.loginURL = element.LoginUrl;
                soDetail.username = element.SignupUsername;
                soDetail.password = element.Password__c;
                soDetail.expityDate = element.ExpirationDate;
                soDetail.status = element.Allocation_status__c
                    ? "In use"
                    : "Not in use";
                scratchOrgToDelete.push(soDetail);
            }
        }
        return scratchOrgToDelete;
    }
}
exports.default = PoolListImpl;
//# sourceMappingURL=poolListImpl.js.map