"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const xml2js = __importStar(require("xml2js"));
const _ = require("lodash");
const util = __importStar(require("util"));
const diffUtil_1 = __importDefault(require("./diffUtil"));
const parser = new xml2js.Parser({
    explicitArray: false,
    valueProcessors: [
        function (name) {
            if (name === "true")
                name = true;
            if (name === "false")
                name = false;
            return name;
        }
    ]
});
class SharingRuleDiff {
    static async generateSharingRulesXml(sharingRuleXml1, sharingRuleXml2, outputFilePath, objectName, destructivePackageObj, resultOutput, isDestructive) {
        const parseString = util.promisify(parser.parseString);
        let sharingRulesObj1 = {};
        let sharingRulesObj2 = {};
        if (sharingRuleXml1 !== "") {
            let parseResult = await parseString(sharingRuleXml1);
            sharingRulesObj1 = parseResult.SharingRules || {};
        }
        if (sharingRuleXml2 !== "") {
            let parseResult = await parseString(sharingRuleXml2);
            sharingRulesObj2 = parseResult.SharingRules || {};
        }
        let addedEditedOrDeleted = SharingRuleDiff.buildSharingRulesObj(sharingRulesObj1, sharingRulesObj2);
        SharingRuleDiff.writeSharingRule(addedEditedOrDeleted.addedEdited, outputFilePath);
        destructivePackageObj = SharingRuleDiff.buildDestructiveChangesObj(addedEditedOrDeleted.deleted, destructivePackageObj, objectName);
        SharingRuleDiff.updateOutput(addedEditedOrDeleted.addedEdited, resultOutput, objectName, "Deploy", outputFilePath);
        if (isDestructive) {
            SharingRuleDiff.updateOutput(addedEditedOrDeleted.deleted, resultOutput, objectName, "Delete", "destructiveChanges.xml");
        }
        return destructivePackageObj;
    }
    static updateOutput(sharingRulesObj, resultOutput, objectName, action, filePath) {
        sharingRulesObj.sharingCriteriaRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingCriteriaRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        sharingRulesObj.sharingOwnerRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingOwnerRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
        sharingRulesObj.sharingTerritoryRules.forEach(elem => {
            resultOutput.push({
                action: action,
                metadataType: "SharingTerritoryRule",
                componentName: `${objectName}.${elem.fullName}`,
                path: filePath
            });
        });
    }
    static ensureArray(sharingObj) {
        let keys = Object.keys(sharingObj);
        keys.forEach(key => {
            if (typeof sharingObj[key] === "object" &&
                !Array.isArray(sharingObj[key]) &&
                key !== "$") {
                sharingObj[key] = [sharingObj[key]];
            }
        });
        return sharingObj;
    }
    static async getMembers(filePath) {
        let fileContent = fs.readFileSync(filePath, "utf8").toString();
        const parseString = util.promisify(parser.parseString);
        let members = {};
        if (fileContent !== "") {
            let parseResult = await parseString(fileContent);
            let sharingRulesObj = parseResult.SharingRules || {};
            if (!_.isNil(sharingRulesObj.sharingCriteriaRules)) {
                if (!Array.isArray(sharingRulesObj.sharingCriteriaRules)) {
                    members["SharingCriteriaRule"] = [
                        sharingRulesObj.sharingCriteriaRules.fullName
                    ];
                }
                else {
                    members["SharingCriteriaRule"] = sharingRulesObj.sharingCriteriaRules.map(sharingRule => {
                        return sharingRule.fullName;
                    });
                }
            }
            if (!_.isNil(sharingRulesObj.sharingOwnerRules)) {
                if (!Array.isArray(sharingRulesObj.sharingOwnerRules)) {
                    members["SharingOwnerRule"] = [
                        sharingRulesObj.sharingOwnerRules.fullName
                    ];
                }
                else {
                    members["SharingOwnerRule"] = sharingRulesObj.sharingOwnerRules.map(sharingRule => {
                        return sharingRule.fullName;
                    });
                }
            }
            if (!_.isNil(sharingRulesObj.sharingTerritoryRules)) {
                if (!Array.isArray(sharingRulesObj.sharingTerritoryRules)) {
                    members["SharingTerritoryRule"] = [
                        sharingRulesObj.sharingTerritoryRules.fullName
                    ];
                }
                else {
                    members["SharingTerritoryRule"] = sharingRulesObj.sharingTerritoryRules.map(sharingRule => {
                        return sharingRule.fullName;
                    });
                }
            }
            if (!_.isNil(sharingRulesObj.sharingGuestRules)) {
                if (!Array.isArray(sharingRulesObj.sharingGuestRules)) {
                    members["SharingGuestRule"] = [
                        sharingRulesObj.sharingGuestRules.fullName
                    ];
                }
                else {
                    members["SharingGuestRule"] = sharingRulesObj.sharingGuestRules.map(sharingRule => {
                        return sharingRule.fullName;
                    });
                }
            }
        }
        return members;
    }
    static buildSharingRulesObj(sharingRuleObj1, sharingRulesObj2) {
        let newSharingRuleObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            sharingCriteriaRules: [],
            sharingOwnerRules: [],
            sharingTerritoryRules: [],
            sharingGuestRules: []
        };
        sharingRuleObj1 = SharingRuleDiff.ensureArray(sharingRuleObj1);
        sharingRulesObj2 = SharingRuleDiff.ensureArray(sharingRulesObj2);
        let deletedSharingObj = {
            $: { xmlns: "http://soap.sforce.com/2006/04/metadata" },
            sharingCriteriaRules: [],
            sharingOwnerRules: [],
            sharingTerritoryRules: [],
            sharingGuestRules: []
        };
        let addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingCriteriaRules, sharingRulesObj2.sharingCriteriaRules, "fullName");
        newSharingRuleObj.sharingCriteriaRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingCriteriaRules = addedDeleted.deleted;
        addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingOwnerRules, sharingRulesObj2.sharingOwnerRules, "fullName");
        newSharingRuleObj.sharingOwnerRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingOwnerRules = addedDeleted.deleted;
        addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingTerritoryRules, sharingRulesObj2.sharingTerritoryRules, "fullName");
        newSharingRuleObj.sharingTerritoryRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingTerritoryRules = addedDeleted.deleted;
        addedDeleted = diffUtil_1.default.getChangedOrAdded(sharingRuleObj1.sharingGuestRules, sharingRulesObj2.sharingGuestRules, "fullName");
        newSharingRuleObj.sharingGuestRules = addedDeleted.addedEdited;
        deletedSharingObj.sharingGuestRules = addedDeleted.deleted;
        return {
            addedEdited: newSharingRuleObj,
            deleted: deletedSharingObj
        };
    }
    static buildDestructiveChangesObj(deletedSharing, destructivePackageObj, objectName) {
        let sharingCriteriaRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingCriteriaRule";
        });
        if (sharingCriteriaRules === undefined &&
            deletedSharing.sharingCriteriaRules !== undefined &&
            deletedSharing.sharingCriteriaRules.length > 0) {
            sharingCriteriaRules = {
                name: "SharingCriteriaRule",
                members: []
            };
            destructivePackageObj.push(sharingCriteriaRules);
        }
        if (deletedSharing.sharingCriteriaRules !== undefined) {
            deletedSharing.sharingCriteriaRules.forEach(elem => {
                sharingCriteriaRules.members.push(objectName + "." + elem.fullName);
            });
        }
        let sharingOwnerRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingOwnerRule";
        });
        if (sharingOwnerRules === undefined &&
            deletedSharing.sharingOwnerRules !== undefined &&
            deletedSharing.sharingOwnerRules.length > 0) {
            sharingOwnerRules = {
                name: "SharingOwnerRule",
                members: []
            };
            destructivePackageObj.push(sharingOwnerRules);
        }
        if (deletedSharing.sharingOwnerRules !== undefined) {
            deletedSharing.sharingOwnerRules.forEach(elem => {
                sharingOwnerRules.members.push(objectName + "." + elem.fullName);
            });
        }
        let sharingTerritoryRules = _.find(destructivePackageObj, function (metaType) {
            return metaType.name === "SharingTerritoryRule";
        });
        if (sharingTerritoryRules === undefined &&
            deletedSharing.sharingTerritoryRules !== undefined &&
            deletedSharing.sharingTerritoryRules.length > 0) {
            sharingTerritoryRules = {
                name: "SharingTerritoryRule",
                members: []
            };
            destructivePackageObj.push(sharingTerritoryRules);
        }
        if (deletedSharing.sharingTerritoryRules !== undefined) {
            deletedSharing.sharingTerritoryRules.forEach(elem => {
                sharingTerritoryRules.members.push(objectName + "." + elem.fullName);
            });
        }
        return destructivePackageObj;
    }
    static writeSharingRule(newSharingRulesObj, outputFilePath) {
        const builder = new xml2js.Builder({
            xmldec: { version: "1.0", encoding: "UTF-8", standalone: null }
        });
        let sharingRulesObj = {
            SharingRules: newSharingRulesObj
        };
        let xml = builder.buildObject(sharingRulesObj);
        fs.writeFileSync(outputFilePath, xml);
    }
}
exports.default = SharingRuleDiff;
//# sourceMappingURL=sharingRuleDiff.js.map