"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const metadataInfo_1 = require("../../../impl/metadata/metadataInfo");
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const metadataFiles_1 = __importDefault(require("../../../impl/metadata/metadataFiles"));
const diffUtil_1 = __importDefault(require("../diff/diffUtil"));
const checkRetrievalStatus_1 = require("../../../utils/checkRetrievalStatus");
const extract_1 = require("../../../utils/extract");
const rimraf = __importStar(require("rimraf"));
const customLabelsDiff_1 = __importDefault(require("../diff/customLabelsDiff"));
const sharingRuleDiff_1 = __importDefault(require("../diff/sharingRuleDiff"));
const workflowDiff_1 = __importDefault(require("../diff/workflowDiff"));
const jsdiff = require("diff");
const { execSync } = require("child_process");
const CRLF_REGEX = /\r\n/;
const LF_REGEX = /\n/;
const unsplitedMetadataExtensions = metadataInfo_1.UNSPLITED_METADATA.filter(elem => {
    return (elem.xmlName !== metadataInfo_1.METADATA_INFO.Profile.xmlName &&
        elem.xmlName !== metadataInfo_1.METADATA_INFO.PermissionSet.xmlName);
}).map(elem => {
    return elem.sourceExtension;
});
class OrgDiffImpl {
    constructor(filesOrFolders, org, addConflictMarkers) {
        this.filesOrFolders = filesOrFolders;
        this.org = org;
        this.addConflictMarkers = addConflictMarkers;
        this.output = [];
    }
    async orgDiff() {
        let packageobj = new Array();
        sfpowerkit_1.SFPowerkit.setStatus("Building package metadata for retrieve");
        this.filesOrFolders.forEach(async (fileOrFolder) => {
            sfpowerkit_1.SFPowerkit.log("Processing " + fileOrFolder, sfpowerkit_1.LoggerLevel.DEBUG);
            fileOrFolder = path.normalize(fileOrFolder);
            let pathExists = fs.existsSync(fileOrFolder);
            if (pathExists) {
                let stats = fs.statSync(fileOrFolder);
                if (stats.isFile()) {
                    //Process File
                    packageobj = await this.buildPackageObj(fileOrFolder, packageobj);
                }
                else if (stats.isDirectory()) {
                    //Process File
                    let files = fileutils_1.default.getAllFilesSync(fileOrFolder);
                    files.forEach(async (oneFile) => {
                        packageobj = await this.buildPackageObj(oneFile, packageobj);
                    });
                }
            }
            else {
                sfpowerkit_1.SFPowerkit.log(`Path ${fileOrFolder} does not exists. `, sfpowerkit_1.LoggerLevel.ERROR);
                throw new Error("Error");
            }
        });
        sfpowerkit_1.SFPowerkit.setStatus("Retrieving metadata");
        await this.retrievePackage(packageobj);
        sfpowerkit_1.SFPowerkit.setStatus("Comparing files");
        this.compare();
        rimraf.sync("temp_sfpowerkit");
        return this.output;
    }
    async buildPackageObj(filePath, packageobj) {
        let matcher = filePath.match(metadataInfo_1.SOURCE_EXTENSION_REGEX);
        let extension = "";
        if (matcher) {
            extension = matcher[0];
        }
        else {
            extension = path.parse(filePath).ext;
        }
        try {
            if (unsplitedMetadataExtensions.includes(extension)) {
                //handle unsplited metadata
                await this.handleUnsplitedMetadatas(filePath, packageobj);
            }
            else {
                let name = metadataInfo_1.MetadataInfo.getMetadataName(filePath, false);
                let member = metadataFiles_1.default.getMemberNameFromFilepath(filePath, name);
                packageobj = diffUtil_1.default.addMemberToPackage(packageobj, name, member);
            }
        }
        catch (err) {
            throw new Error(err + ",Error file path : " + filePath);
        }
        return packageobj;
    }
    async handleUnsplitedMetadatas(filePath, packageobj) {
        if (filePath.endsWith(metadataInfo_1.METADATA_INFO.CustomLabels.sourceExtension)) {
            let members = await customLabelsDiff_1.default.getMembers(filePath);
            packageobj.push({
                name: "CustomLabel",
                members: members
            });
        }
        if (filePath.endsWith(metadataInfo_1.METADATA_INFO.SharingRules.sourceExtension)) {
            let name = metadataInfo_1.MetadataInfo.getMetadataName(filePath, false);
            let objectName = metadataFiles_1.default.getMemberNameFromFilepath(filePath, name);
            let members = await sharingRuleDiff_1.default.getMembers(filePath);
            Object.keys(members).forEach(key => {
                packageobj.push({
                    name: key,
                    members: members[key].map(elem => {
                        return objectName + "." + elem;
                    })
                });
            });
        }
        if (filePath.endsWith(metadataInfo_1.METADATA_INFO.Workflow.sourceExtension)) {
            let name = metadataInfo_1.MetadataInfo.getMetadataName(filePath, false);
            let objectName = metadataFiles_1.default.getMemberNameFromFilepath(filePath, name);
            let members = await workflowDiff_1.default.getMembers(filePath);
            Object.keys(members).forEach(key => {
                packageobj.push({
                    name: key,
                    members: members[key].map(elem => {
                        return objectName + "." + elem;
                    })
                });
            });
        }
    }
    compare() {
        // let fetchedFiles = FileUtils.getAllFilesSync(`./temp_sfpowerkit/mdapi`, "");
        let fetchedFiles = fileutils_1.default.getAllFilesSync(`./temp_sfpowerkit/force-app`, "");
        this.filesOrFolders.forEach(fileOrFolder => {
            fileOrFolder = path.normalize(fileOrFolder);
            let pathExists = fs.existsSync(fileOrFolder);
            if (pathExists) {
                let stats = fs.statSync(fileOrFolder);
                if (stats.isFile()) {
                    //Process File
                    this.processFile(fileOrFolder, fetchedFiles);
                }
                else if (stats.isDirectory()) {
                    //Read files in directory
                    let files = fileutils_1.default.getAllFilesSync(fileOrFolder, "");
                    files.forEach(oneFile => {
                        //process file
                        this.processFile(oneFile, fetchedFiles);
                    });
                }
            }
        });
    }
    processFile(localFile, fetchedFiles) {
        sfpowerkit_1.SFPowerkit.log("Compare:  Processing " + localFile, sfpowerkit_1.LoggerLevel.DEBUG);
        let metaType = metadataInfo_1.MetadataInfo.getMetadataName(localFile, false);
        let member = metadataFiles_1.default.getMemberNameFromFilepath(localFile, metaType);
        // let extension = path.parse(localFile).ext;
        let cmpPath = path.parse(localFile).base;
        let metadataDescribe = metadataInfo_1.METADATA_INFO[metaType];
        const staticResourceRegExp = new RegExp("staticresources");
        if (metadataDescribe.inFolder || staticResourceRegExp.test(localFile)) {
            let folderName = "staticresources";
            if (metadataDescribe.inFolder) {
                folderName = metadataDescribe.directoryName;
            }
            let baseIndex = localFile.indexOf(folderName) + folderName.length;
            cmpPath = localFile.substring(baseIndex + 1);
        }
        // find the files
        let foundFile = fetchedFiles.find(fetchFile => {
            let fetchedMetaType = metadataInfo_1.MetadataInfo.getMetadataName(fetchFile, false);
            let fetchedMember = metadataFiles_1.default.getMemberNameFromFilepath(fetchFile, fetchedMetaType);
            //let fetchedExtension = path.parse(fetchFile).ext;
            let fetchedCmpPath = path.parse(fetchFile).base;
            let fetchedMetadataDescribe = metadataInfo_1.METADATA_INFO[fetchedMetaType];
            if (fetchedMetadataDescribe.inFolder ||
                staticResourceRegExp.test(fetchFile)) {
                let fetchedFolderName = "staticresources";
                if (fetchedMetadataDescribe.inFolder) {
                    fetchedFolderName = fetchedMetadataDescribe.directoryName;
                }
                let fetchedBaseIndex = fetchFile.indexOf(fetchedFolderName) + fetchedFolderName.length;
                fetchedCmpPath = fetchFile.substring(fetchedBaseIndex + 1);
            }
            return (fetchedMetaType === metaType &&
                member === fetchedMember &&
                cmpPath === fetchedCmpPath);
        });
        if (foundFile !== undefined) {
            let contentLocalFile = fs.readFileSync(localFile, "utf8");
            let contentFetchedFile = fs.readFileSync(foundFile, "utf8");
            //Normalise line ending on windows
            let matcherLocal = contentLocalFile.match(CRLF_REGEX);
            let matcherFetched = contentFetchedFile.match(CRLF_REGEX);
            let lineEnd = "\n";
            if (matcherLocal && !matcherFetched) {
                lineEnd = matcherLocal[0];
                contentFetchedFile = contentFetchedFile.split(LF_REGEX).join(lineEnd);
            }
            if (!contentLocalFile.endsWith(lineEnd) &&
                contentFetchedFile.endsWith(lineEnd)) {
                contentFetchedFile = contentFetchedFile.substr(0, contentFetchedFile.lastIndexOf(lineEnd));
            }
            if (contentLocalFile.endsWith(lineEnd) &&
                !contentFetchedFile.endsWith(lineEnd)) {
                contentFetchedFile = contentFetchedFile + lineEnd;
            }
            let diffResult = jsdiff.diffLines(contentLocalFile, contentFetchedFile);
            //Process the diff result add add conflict marker on the files
            this.processResult(localFile, diffResult);
        }
        else {
            this.output.push({
                status: "Local Added / Remote Deleted",
                metadataType: metaType,
                componentName: member,
                path: localFile
            });
        }
    }
    processResult(filePath, diffResult) {
        let content = "";
        let firstChunkProcessed = false;
        let changedLocaly = false;
        let changedRemote = false;
        let conflict = false;
        for (let i = 0; i < diffResult.length; i++) {
            let result = diffResult[i];
            let index = i;
            let originalArray = diffResult;
            if (result.removed) {
                if (firstChunkProcessed) {
                    content = content + `${result.value}>>>>>>> Remote:${filePath}\n`;
                    firstChunkProcessed = false;
                    conflict = true;
                }
                else if ((originalArray.length > index + 1 &&
                    originalArray[index + 1].added == undefined) ||
                    index + 1 === originalArray.length) {
                    //Line added locally and remove remote
                    content =
                        content +
                            `<<<<<<< Local:${filePath}\n${result.value}=======\n>>>>>>> Remote:${filePath}\n`;
                    firstChunkProcessed = false;
                    changedLocaly = true;
                }
                else {
                    content =
                        content + `<<<<<<< Local:${filePath}\n${result.value}=======\n`;
                    firstChunkProcessed = true;
                    conflict = true;
                }
            }
            else if (result.added) {
                if (firstChunkProcessed) {
                    content = content + `${result.value}>>>>>>> Remote:${filePath}\n`;
                    firstChunkProcessed = false;
                    conflict = true;
                }
                else if ((originalArray.length > index + 1 &&
                    originalArray[index + 1].removed == undefined &&
                    originalArray[index + 1].added == undefined) ||
                    index + 1 === originalArray.length) {
                    //Line added locally and remove remote
                    content =
                        content +
                            `<<<<<<< Local:${filePath}\n=======\n${result.value}>>>>>>> Remote:${filePath}\n`;
                    firstChunkProcessed = false;
                    changedRemote = true;
                }
                else {
                    //This should never happen
                    content =
                        content + `<<<<<<< Local:${filePath}\n${result.value}=======\n`;
                    firstChunkProcessed = true;
                    conflict = true;
                }
            }
            else {
                content = content + result.value;
            }
        }
        if (this.addConflictMarkers) {
            fs.writeFileSync(filePath, content);
        }
        let status = "No Change";
        if (conflict || (changedLocaly && changedRemote)) {
            status = "Conflict";
        }
        else if (changedRemote) {
            status = "Remote Change";
        }
        else if (changedLocaly) {
            status = "Local Change";
        }
        let metaType = metadataInfo_1.MetadataInfo.getMetadataName(filePath, false);
        let member = metadataFiles_1.default.getMemberNameFromFilepath(filePath, metaType);
        if (conflict || changedLocaly || changedRemote) {
            this.output.push({
                status: status,
                metadataType: metaType,
                componentName: member,
                path: filePath
            });
        }
    }
    async retrievePackage(packageObj) {
        sfpowerkit_1.SFPowerkit.log("Clear temp folder ", sfpowerkit_1.LoggerLevel.INFO);
        rimraf.sync("temp_sfpowerkit");
        const apiversion = await this.org.getConnection().retrieveMaxApiVersion();
        let retrieveRequest = {
            apiVersion: apiversion
        };
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: packageObj
        };
        // if(!this.flags.json)
        // this.ux.logJson(retrieveRequest);
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        sfpowerkit_1.SFPowerkit.log("Retrieve request sent ", sfpowerkit_1.LoggerLevel.INFO);
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        sfpowerkit_1.SFPowerkit.setStatus("Retrieving metadata | WAITING for retrieve request ");
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId, false);
        sfpowerkit_1.SFPowerkit.setStatus("Retrieving metadata");
        sfpowerkit_1.SFPowerkit.log("Retrieve completed. Writing retrieved metadata to disk ", sfpowerkit_1.LoggerLevel.DEBUG);
        if (!metadata_retrieve_result.zipFile)
            throw new Error("Error while retrieveing metadata");
        var zipFileName = "temp_sfpowerkit/unpackaged.zip";
        fs.mkdirSync("temp_sfpowerkit");
        fs.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64"
        });
        sfpowerkit_1.SFPowerkit.log("Extracting retrieved metadata ", sfpowerkit_1.LoggerLevel.DEBUG);
        await extract_1.extract(`./temp_sfpowerkit/unpackaged.zip`, "temp_sfpowerkit/mdapi");
        let maxApiVersion = await this.org.retrieveMaxApiVersion();
        fs.mkdirSync("temp_sfpowerkit/force-app");
        sfpowerkit_1.SFPowerkit.log("Converting retrieved metadata to dx format", sfpowerkit_1.LoggerLevel.INFO);
        let sfdxProjectJson = `{
        "packageDirectories": [
          {
            "path": "force-app",
            "default": true
          }
        ],
        "namespace": "",
        "sfdcLoginUrl": "https://login.salesforce.com",
        "sourceApiVersion": "${maxApiVersion}"
      }`;
        fs.writeFileSync("temp_sfpowerkit/sfdx-project.json", sfdxProjectJson);
        execSync("sfdx force:mdapi:convert -r mdapi -d force-app", {
            cwd: "temp_sfpowerkit"
        });
        //Should remove the mdapi folder
        rimraf.sync("temp_sfpowerkit/mdapi");
        rimraf.sync("temp_sfpowerkit/unpackaged.zip");
    }
}
exports.default = OrgDiffImpl;
//# sourceMappingURL=orgDiffImpl.js.map