"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const retrieveMetadata_1 = require("../../../utils/retrieveMetadata");
const core_1 = require("@salesforce/core");
const path = __importStar(require("path"));
const fs = __importStar(require("fs"));
const metadataInfo_1 = require("../../../impl/metadata/metadataInfo");
const profileRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/profileRetriever"));
const profileWriter_1 = __importDefault(require("../../../impl/metadata/writer/profileWriter"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const metadataFiles_1 = __importDefault(require("../../../impl/metadata/metadataFiles"));
const diff_match_patch_1 = require("diff-match-patch");
require("diff-match-patch-line-and-word"); // import globally to  enhanse the class
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const rimraf = __importStar(require("rimraf"));
const progressBar_1 = require("../../../../src/ui/progressBar");
const dmp = new diff_match_patch_1.diff_match_patch();
const CRLF_REGEX = /\r\n/;
const LF_REGEX = /\n/;
class ProfileDiffImpl {
    constructor(profileList, sourceOrgStr, targetOrg, outputFolder) {
        this.profileList = profileList;
        this.sourceOrgStr = sourceOrgStr;
        this.targetOrg = targetOrg;
        this.outputFolder = outputFolder;
        this.sourceOrg = null;
        this.output = [];
        this.sourceLabel = "Local";
        this.targetLabel = "Remote";
        this.targetLabel = this.targetOrg.getConnection().getUsername();
    }
    async diff() {
        sfpowerkit_1.SFPowerkit.log("Profile diff start. ", core_1.LoggerLevel.INFO);
        if (this.outputFolder) {
            rimraf.sync(this.outputFolder);
        }
        let profileSource = null;
        //let profileXmlMapPromise: Promise<string[]> = null;
        if (this.sourceOrgStr) {
            sfpowerkit_1.SFPowerkit.log("Creating source org ", core_1.LoggerLevel.INFO);
            this.sourceOrg = await core_1.Org.create({
                aliasOrUsername: this.sourceOrgStr,
                isDevHub: false
            });
        }
        if ((!this.profileList || this.profileList.length === 0) &&
            this.sourceOrgStr) {
            this.sourceLabel = this.sourceOrg.getConnection().getUsername();
            sfpowerkit_1.SFPowerkit.log("No profile provided, loading all profiles from source org. ", core_1.LoggerLevel.INFO);
            let conn = this.sourceOrg.getConnection();
            let profileNamesPromise = retrieveMetadata_1.retrieveMetadata([{ type: "Profile", folder: null }], conn);
            profileSource = profileNamesPromise.then(profileNames => {
                return this.retrieveProfiles(profileNames, this.sourceOrg);
            });
        }
        else {
            sfpowerkit_1.SFPowerkit.log("Reading profiles from file system. ", core_1.LoggerLevel.INFO);
            let srcFolders = await sfpowerkit_1.SFPowerkit.getProjectDirectories();
            let metadataFiles = new metadataFiles_1.default();
            sfpowerkit_1.SFPowerkit.log("Source Folders are", core_1.LoggerLevel.DEBUG);
            for (let i = 0; i < srcFolders.length; i++) {
                let srcFolder = srcFolders[i];
                let normalizedPath = path.join(process.cwd(), srcFolder);
                metadataFiles.loadComponents(normalizedPath);
            }
            if (!this.profileList || this.profileList.length === 0) {
                this.profileList = metadataInfo_1.METADATA_INFO.Profile.files;
            }
            else {
                this.profileList = this.profileList.map(profilename => {
                    const foundFile = metadataInfo_1.METADATA_INFO.Profile.files.find(file => {
                        const apiName = metadataFiles_1.default.getFullApiName(file);
                        return apiName === profilename;
                    });
                    if (!foundFile) {
                        sfpowerkit_1.SFPowerkit.log("No profile found with name  " + profilename, core_1.LoggerLevel.INFO);
                    }
                    return foundFile;
                });
                this.profileList = this.profileList.filter(file => {
                    return file !== undefined;
                });
            }
            if (!this.profileList || this.profileList.length === 0) {
                sfpowerkit_1.SFPowerkit.log("No profile to process ", core_1.LoggerLevel.INFO);
                return null;
            }
            if (!this.outputFolder) {
                this.outputFolder = path.dirname(this.profileList[0]);
            }
            let profilesMap = [];
            let progressBar = new progressBar_1.ProgressBar();
            progressBar.create("Reading from File System ", "Profiles", core_1.LoggerLevel.FATAL);
            progressBar.start(this.profileList.length);
            for (let i = 0; i < this.profileList.length; i++) {
                let profilepath = this.profileList[i];
                sfpowerkit_1.SFPowerkit.log("Reading profile from path " + profilepath, core_1.LoggerLevel.DEBUG);
                let profileXml = fs.readFileSync(profilepath);
                let profileName = path.basename(profilepath, metadataInfo_1.METADATA_INFO.Profile.sourceExtension);
                profilesMap.push({
                    [profileName]: profileXml.toString()
                });
                progressBar.increment(1);
            }
            profileSource = new Promise((resolve, reject) => {
                resolve(profilesMap);
            });
            progressBar.stop();
        }
        if (!fs.existsSync(this.outputFolder)) {
            console.log("Creattin output diff " + this.outputFolder);
            fileutils_1.default.mkDirByPathSync(this.outputFolder);
        }
        //REtrieve profiles from target
        return profileSource.then(profilesSourceMap => {
            let profileNames = [];
            profilesSourceMap.forEach(profileXml => {
                profileNames.push(...Object.keys(profileXml));
            });
            let targetConn = this.targetOrg.getConnection();
            let profileNamesPromise = retrieveMetadata_1.retrieveMetadata([{ type: "Profile", folder: null }], targetConn);
            let profileTarget = profileNamesPromise
                .then(targetProfileNames => {
                let profileToRetrieveinTarget = profileNames.filter(oneProfile => {
                    return targetProfileNames.includes(oneProfile);
                });
                return this.retrieveProfiles(profileToRetrieveinTarget, this.targetOrg);
            })
                .catch(error => {
                console.log(error.message);
                return [];
            });
            return profileTarget
                .then(profilesTargetMap => {
                sfpowerkit_1.SFPowerkit.log("Handling diff ", core_1.LoggerLevel.INFO);
                let progressBar = new progressBar_1.ProgressBar().create("Diff processing ", "Profiles", core_1.LoggerLevel.INFO);
                progressBar.start(profilesSourceMap.length);
                for (let i = 0; i < profilesSourceMap.length; i++) {
                    let sourceProfileXml = profilesSourceMap[i];
                    let sourceKeys = Object.keys(sourceProfileXml);
                    let sourceProfileName = sourceKeys[0];
                    let targetProfileXml = profilesTargetMap.find(targetProfile => {
                        let targetKeys = Object.keys(targetProfile);
                        let targetProfileName = targetKeys[0];
                        return targetProfileName === sourceProfileName;
                    });
                    sfpowerkit_1.SFPowerkit.log("Processing profile " + sourceProfileName, core_1.LoggerLevel.DEBUG);
                    let sourceContent = sourceProfileXml[sourceProfileName];
                    let targetContent = "";
                    if (targetProfileXml) {
                        targetContent = targetProfileXml[sourceProfileName];
                    }
                    let filePath = this.outputFolder +
                        path.sep +
                        sourceProfileName +
                        metadataInfo_1.METADATA_INFO.Profile.sourceExtension;
                    sfpowerkit_1.SFPowerkit.log("Processing diff for profile " + sourceProfileName, core_1.LoggerLevel.DEBUG);
                    this.processDiff(filePath, sourceContent, targetContent);
                    progressBar.increment(1);
                }
                /*
                profilesSourceMap.forEach(sourceProfileXml => {
                  
                });
                */
                progressBar.stop();
                return this.output;
            })
                .catch(error => {
                console.log(error.message);
            });
        });
    }
    async retrieveProfiles(profileNames, retrieveOrg) {
        let i, j, chunk = 10, temparray;
        let profileRetriever = new profileRetriever_1.default(retrieveOrg, false);
        let retrievePromises = [];
        let connection = retrieveOrg.getConnection();
        let progressBar = new progressBar_1.ProgressBar().create(`Retrieving From ${connection.getUsername()}`, "Profiles", core_1.LoggerLevel.INFO);
        progressBar.start(profileNames.length);
        for (i = 0, j = profileNames.length; i < j; i += chunk) {
            temparray = profileNames.slice(i, i + chunk);
            let metadataListPromise = profileRetriever.loadProfiles(temparray, connection);
            retrievePromises.push(metadataListPromise
                .then(metadataList => {
                let profileWriter = new profileWriter_1.default();
                let profilesXmls = [];
                for (let count = 0; count < metadataList.length; count++) {
                    //console.log(metadataList[count]);
                    let profileObj = metadataList[count];
                    let profileXml = profileWriter.toXml(profileObj);
                    profilesXmls.push({
                        [profileObj.fullName]: profileXml
                    });
                    progressBar.increment(1);
                }
                return profilesXmls;
            })
                .catch(error => {
                console.error(error.message);
                progressBar.stop();
                return [];
            }));
        }
        return Promise.all(retrievePromises)
            .then(metadataList => {
            let profiles = [];
            metadataList.forEach(elem => {
                profiles.push(...elem);
            });
            progressBar.stop();
            return profiles;
        })
            .catch(error => {
            console.error(error.message);
            progressBar.stop();
            return [];
        });
    }
    processDiff(filePath, contentSource, contentTarget) {
        let lineEnd = "\n";
        let content = "";
        let changedLocaly = false;
        let changedRemote = false;
        let conflict = false;
        //Normalise line ending on windows
        let matcherLocal = contentSource.match(CRLF_REGEX);
        let matcherFetched = contentTarget.match(CRLF_REGEX);
        if (matcherLocal && !matcherFetched) {
            lineEnd = matcherLocal[0];
            contentTarget = contentTarget.split(LF_REGEX).join(lineEnd);
        }
        if (!contentSource.endsWith(lineEnd) && contentTarget.endsWith(lineEnd)) {
            contentTarget = contentTarget.substr(0, contentTarget.lastIndexOf(lineEnd));
        }
        if (contentSource.endsWith(lineEnd) && !contentTarget.endsWith(lineEnd)) {
            contentTarget = contentTarget + lineEnd;
        }
        sfpowerkit_1.SFPowerkit.log("Running diff", core_1.LoggerLevel.DEBUG);
        //let diffResult = jsdiff.diffLines(contentSource, contentTarget);
        const diffResult = dmp.diff_lineMode(contentSource, contentTarget);
        sfpowerkit_1.SFPowerkit.log("Diff run completed. Processing result", core_1.LoggerLevel.DEBUG);
        for (let i = 0; i < diffResult.length; i++) {
            let result = diffResult[i];
            let index = i;
            let originalArray = diffResult;
            let nextIndex = index + 1;
            let nextElem = undefined;
            if (originalArray.length >= nextIndex) {
                nextElem = originalArray[nextIndex];
            }
            let value = result[1];
            let status = result[0];
            if (status === -1) {
                if (!value.endsWith(lineEnd)) {
                    value = value + lineEnd;
                }
                if (nextElem !== undefined) {
                    if (nextElem[0] === 0) {
                        content =
                            content +
                                `<<<<<<< ${this.sourceLabel}:${filePath}\n${value}=======\n>>>>>>> ${this.targetLabel}:${filePath}\n`;
                        changedLocaly = true;
                    }
                    else if (nextElem[0] === 1) {
                        content =
                            content +
                                `<<<<<<< ${this.sourceLabel}:${filePath}\n${value}=======\n`;
                        conflict = true;
                    }
                }
                else {
                    content =
                        content +
                            `<<<<<<< ${this.sourceLabel}:${filePath}\n${value}=======\n>>>>>>> ${this.targetLabel}:${filePath}\n`;
                    changedLocaly = true;
                }
            }
            else if (status === 1) {
                if (conflict) {
                    content =
                        content + `${value}>>>>>>> ${this.targetLabel}:${filePath}\n`;
                    conflict = true;
                }
                else {
                    content =
                        content +
                            `<<<<<<< ${this.sourceLabel}:${filePath}\n=======\n${value}>>>>>>> ${this.targetLabel}:${filePath}\n`;
                    changedRemote = true;
                }
            }
            else {
                content = content + value;
            }
        }
        sfpowerkit_1.SFPowerkit.log("Result processed", core_1.LoggerLevel.DEBUG);
        fs.writeFileSync(filePath, content);
        let status = "No Change";
        if (conflict || (changedLocaly && changedRemote)) {
            status = "Conflict";
        }
        else if (changedRemote) {
            status = "Remote Change";
        }
        else if (changedLocaly) {
            status = "Local Change";
        }
        let metaType = metadataInfo_1.MetadataInfo.getMetadataName(filePath, false);
        let member = metadataFiles_1.default.getMemberNameFromFilepath(filePath, metaType);
        if (conflict || changedLocaly || changedRemote) {
            this.output.push({
                status: status,
                metadataType: metaType,
                componentName: member,
                path: filePath
            });
        }
    }
}
exports.default = ProfileDiffImpl;
//# sourceMappingURL=profileDiff.js.map