"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const sfpowerkit_1 = require("../../../sfpowerkit");
const metadataFiles_1 = __importDefault(require("../../metadata/metadataFiles"));
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const xml2js = __importStar(require("xml2js"));
const metadataInfo_1 = require("../../metadata/metadataInfo");
const customApplicationRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/customApplicationRetriever"));
const apexClassRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/apexClassRetriever"));
const fieldRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/fieldRetriever"));
const layoutRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/layoutRetriever"));
const recordTypeRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/recordTypeRetriever"));
const entityDefinitionRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/entityDefinitionRetriever"));
const apexPageRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/apexPageRetriever"));
const tabDefinitionRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/tabDefinitionRetriever"));
const userLicenseRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/userLicenseRetriever"));
const userPermissionBuilder_1 = __importDefault(require("../../../impl/metadata/builder/userPermissionBuilder"));
const util = __importStar(require("util"));
const _ = __importStar(require("lodash"));
const profileActions_1 = __importDefault(require("./profileActions"));
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const profileWriter_1 = __importDefault(require("../../../impl/metadata/writer/profileWriter"));
const core_1 = require("@salesforce/core");
const externalDataSourceRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/externalDataSourceRetriever"));
const flowRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/flowRetriever"));
const customPermissionRetriever_1 = __importDefault(require("../../../impl/metadata/retriever/customPermissionRetriever"));
const nonArayProperties = [
    "custom",
    "description",
    "fullName",
    "userLicense",
    "$"
];
class ProfileReconcile extends profileActions_1.default {
    async reconcile(srcFolders, profileList, destFolder) {
        if (!_.isNil(destFolder)) {
            fileutils_1.default.mkDirByPathSync(destFolder);
        }
        if (_.isNil(srcFolders) || srcFolders.length === 0) {
            srcFolders = await sfpowerkit_1.SFPowerkit.getProjectDirectories();
        }
        let result = [];
        this.metadataFiles = new metadataFiles_1.default();
        srcFolders.forEach(srcFolder => {
            let normalizedPath = path.join(process.cwd(), srcFolder);
            this.metadataFiles.loadComponents(normalizedPath);
        });
        profileList = profileList.map(element => {
            return element + metadataInfo_1.METADATA_INFO.Profile.sourceExtension;
        });
        if (!metadataFiles_1.default.sourceOnly) {
            await this.profileRetriever.loadSupportedPermissions();
        }
        for (let count = 0; count < metadataInfo_1.METADATA_INFO.Profile.files.length; count++) {
            let profileComponent = metadataInfo_1.METADATA_INFO.Profile.files[count];
            if (profileList.length == 0 ||
                profileList.includes(path.basename(profileComponent))) {
                sfpowerkit_1.SFPowerkit.log("Reconciling profile " + path.basename(profileComponent), core_1.LoggerLevel.INFO);
                let profileXmlString = fs.readFileSync(profileComponent);
                const parser = new xml2js.Parser({ explicitArray: true });
                const parseString = util.promisify(parser.parseString);
                let parseResult = await parseString(profileXmlString);
                let profileWriter = new profileWriter_1.default();
                let profileObj = profileWriter.toProfile(parseResult.Profile); // as Profile
                profileObj = await this.removePermissions(profileObj);
                if (!metadataFiles_1.default.sourceOnly) {
                    //Manage licences
                    let licenceUtils = userLicenseRetriever_1.default.getInstance(this.org);
                    const isSupportedLicence = await licenceUtils.userLicenseExists(profileObj.userLicense);
                    if (!isSupportedLicence) {
                        delete profileObj.userLicense;
                    }
                }
                // remove unsupported userPermission
                let unsupportedLicencePermissions = this.profileRetriever.getUnsupportedLicencePermissions(profileObj.userLicense);
                if (profileObj.userPermissions != null &&
                    profileObj.userPermissions.length > 0) {
                    profileObj.userPermissions = profileObj.userPermissions.filter(permission => {
                        let supported = !unsupportedLicencePermissions.includes(permission.name);
                        return supported;
                    });
                }
                //IS sourceonly, use ignorePermission set in sfdxProject.json file
                if (metadataFiles_1.default.sourceOnly) {
                    let pluginConfig = await sfpowerkit_1.SFPowerkit.getConfig();
                    let ignorePermissions = pluginConfig.ignoredPermissions || [];
                    if (profileObj.userPermissions !== undefined &&
                        profileObj.userPermissions.length > 0) {
                        profileObj.userPermissions = profileObj.userPermissions.filter(permission => {
                            let supported = !ignorePermissions.includes(permission.name);
                            return supported;
                        });
                    }
                }
                else {
                    if (profileObj.userPermissions !== undefined &&
                        profileObj.userPermissions.length > 0) {
                        //Remove permission that are not present in the target org
                        profileObj.userPermissions = profileObj.userPermissions.filter(permission => {
                            let supported = this.profileRetriever.supportedPermissions.includes(permission.name);
                            return supported;
                        });
                    }
                }
                //UserPermissionUtils.addPermissionDependencies(profileObj);
                let isCustom = "" + profileObj.custom;
                if (isCustom == "false") {
                    delete profileObj.userPermissions;
                }
                //this.handleViewAllDataPermission(profileObj);
                //this.handleInstallPackagingPermission(profileObj);
                //this.handleQueryAllFilesPermission(profileObj);
                userPermissionBuilder_1.default.handlePermissionDependency(profileObj, this.profileRetriever.supportedPermissions);
                //Delete eampty arrays
                for (let key in profileObj) {
                    if (Array.isArray(profileObj[key])) {
                        //All top element must be arays exept non arrayProperties
                        if (!nonArayProperties.includes(key) &&
                            profileObj[key].length === 0) {
                            delete profileObj[key];
                        }
                    }
                }
                let builder = new xml2js.Builder({ rootName: "Profile" });
                let xml = builder.buildObject(profileObj);
                let outputFile = profileComponent;
                if (!_.isNil(destFolder)) {
                    outputFile = path.join(destFolder, path.basename(profileComponent));
                }
                fs.writeFileSync(outputFile, xml);
                result.push(outputFile);
            }
        }
        return result;
    }
    async reconcileApp(profileObj) {
        let utils = customApplicationRetriever_1.default.getInstance(this.org);
        if (profileObj.applicationVisibilities !== undefined) {
            let validArray = [];
            for (let i = 0; i < profileObj.applicationVisibilities.length; i++) {
                let cmpObj = profileObj.applicationVisibilities[i];
                let exist = await utils.appExists(cmpObj.application);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Application Visiblitilties reduced from ${profileObj.applicationVisibilities.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.applicationVisibilities = validArray;
        }
        return profileObj;
    }
    async reconcileClasses(profileObj) {
        let utils = apexClassRetriever_1.default.getInstance(this.org);
        if (profileObj.classAccesses !== undefined) {
            if (!Array.isArray(profileObj.classAccesses)) {
                profileObj.classAccesses = [profileObj.classAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.classAccesses.length; i++) {
                let cmpObj = profileObj.classAccesses[i];
                let exists = await utils.classExists(cmpObj.apexClass);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Class Access reduced from ${profileObj.classAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.classAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileFields(profileObj) {
        let utils = fieldRetriever_1.default.getInstance(this.org);
        if (profileObj.fieldLevelSecurities !== undefined) {
            if (!Array.isArray(profileObj.fieldLevelSecurities)) {
                profileObj.fieldLevelSecurities = [profileObj.fieldLevelSecurities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.fieldLevelSecurities.length; i++) {
                let cmpObj = profileObj.fieldLevelSecurities[i];
                let exists = await utils.fieldExist(cmpObj.field);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Fields Level Security reduced from ${profileObj.fieldLevelSecurities.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.fieldLevelSecurities = validArray;
        }
        if (profileObj.fieldPermissions !== undefined) {
            if (!Array.isArray(profileObj.fieldPermissions)) {
                profileObj.fieldPermissions = [profileObj.fieldPermissions];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.fieldPermissions.length; i++) {
                let cmpObj = profileObj.fieldPermissions[i];
                let exists = await utils.fieldExist(cmpObj.field);
                if (exists) {
                    validArray.push(cmpObj);
                }
            }
            profileObj.fieldPermissions = validArray;
        }
        return profileObj;
    }
    async reconcileLayouts(profileObj) {
        let utils = layoutRetriever_1.default.getInstance(this.org);
        let rtUtils = recordTypeRetriever_1.default.getInstance(this.org);
        if (profileObj.layoutAssignments !== undefined) {
            var validArray = [];
            for (let count = 0; count < profileObj.layoutAssignments.length; count++) {
                let cmpObj = profileObj.layoutAssignments[count];
                let exist = (await utils.layoutExists(cmpObj.layout)) &&
                    (_.isNil(cmpObj.recordType) ||
                        (await rtUtils.recordTypeExists(cmpObj.recordType)));
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Layout Assignnments reduced from ${profileObj.layoutAssignments.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.layoutAssignments = validArray;
        }
        return profileObj;
    }
    async reconcileObjects(profileObj) {
        let utils = entityDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.objectPermissions !== undefined) {
            if (!Array.isArray(profileObj.objectPermissions)) {
                profileObj.objectPermissions = [profileObj.objectPermissions];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.objectPermissions.length; i++) {
                let cmpObj = profileObj.objectPermissions[i];
                let exist = await utils.existObjectPermission(cmpObj.object);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Object Permissions reduced from ${profileObj.objectPermissions.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.objectPermissions = validArray;
        }
        return profileObj;
    }
    async reconcileCustomMetadata(profileObj) {
        let utils = entityDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.customMetadataTypeAccesses !== undefined) {
            if (!Array.isArray(profileObj.customMetadataTypeAccesses)) {
                profileObj.customMetadataTypeAccesses = [
                    profileObj.customMetadataTypeAccesses
                ];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.customMetadataTypeAccesses.length; i++) {
                let cmpCM = profileObj.customMetadataTypeAccesses[i];
                let exist = await utils.existCustomMetadata(cmpCM.name);
                if (exist) {
                    validArray.push(cmpCM);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`CustomMetadata Access reduced from ${profileObj.customMetadataTypeAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.customMetadataTypeAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileCustomSettins(profileObj) {
        let utils = entityDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.customSettingAccesses !== undefined) {
            if (!Array.isArray(profileObj.customSettingAccesses)) {
                profileObj.customSettingAccesses = [profileObj.customSettingAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.customSettingAccesses.length; i++) {
                let cmpCS = profileObj.customSettingAccesses[i];
                let exist = await utils.existCustomMetadata(cmpCS.name);
                if (exist) {
                    validArray.push(cmpCS);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`CustomSettings Access reduced from ${profileObj.customSettingAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.customSettingAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileExternalDataSource(profileObj) {
        let utils = externalDataSourceRetriever_1.default.getInstance(this.org);
        if (profileObj.externalDataSourceAccesses !== undefined) {
            if (!Array.isArray(profileObj.externalDataSourceAccesses)) {
                profileObj.externalDataSourceAccesses = [
                    profileObj.externalDataSourceAccesses
                ];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.externalDataSourceAccesses.length; i++) {
                let dts = profileObj.externalDataSourceAccesses[i];
                let exist = await utils.externalDataSourceExists(dts.externalDataSource);
                if (exist) {
                    validArray.push(dts);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`ExternalDataSource Access reduced from ${profileObj.externalDataSourceAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.externalDataSourceAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileFlow(profileObj) {
        let utils = flowRetriever_1.default.getInstance(this.org);
        if (profileObj.flowAccesses !== undefined) {
            if (!Array.isArray(profileObj.flowAccesses)) {
                profileObj.flowAccesses = [profileObj.flowAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.flowAccesses.length; i++) {
                let flow = profileObj.flowAccesses[i];
                let exist = await utils.flowExists(flow.flow);
                if (exist) {
                    validArray.push(flow);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Flow Access reduced from ${profileObj.flowAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.flowAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileCustomPermission(profileObj) {
        let utils = customPermissionRetriever_1.default.getInstance(this.org);
        if (profileObj.customPermissions !== undefined) {
            if (!Array.isArray(profileObj.customPermissions)) {
                profileObj.customPermissions = [profileObj.customPermissions];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.customPermissions.length; i++) {
                let customPermission = profileObj.customPermissions[i];
                let exist = await utils.customPermissionExists(customPermission.name);
                if (exist) {
                    validArray.push(customPermission);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`CustomPermission reduced from ${profileObj.customPermissions.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.customPermissions = validArray;
        }
        return profileObj;
    }
    async reconcilePages(profileObj) {
        let utils = apexPageRetriever_1.default.getInstance(this.org);
        if (profileObj.pageAccesses !== undefined) {
            if (!Array.isArray(profileObj.pageAccesses)) {
                profileObj.pageAccesses = [profileObj.pageAccesses];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.pageAccesses.length; i++) {
                let cmpObj = profileObj.pageAccesses[i];
                let exist = await utils.pageExists(cmpObj.apexPage);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Page Access Permissions reduced from ${profileObj.pageAccesses.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.pageAccesses = validArray;
        }
        return profileObj;
    }
    async reconcileRecordTypes(profileObj) {
        let utils = recordTypeRetriever_1.default.getInstance(this.org);
        if (profileObj.recordTypeVisibilities !== undefined) {
            if (!Array.isArray(profileObj.recordTypeVisibilities)) {
                profileObj.recordTypeVisibilities = [profileObj.recordTypeVisibilities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.recordTypeVisibilities.length; i++) {
                let cmpObj = profileObj.recordTypeVisibilities[i];
                let exist = await utils.recordTypeExists(cmpObj.recordType);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Record Type Visibilities reduced from ${profileObj.recordTypeVisibilities.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.recordTypeVisibilities = validArray;
        }
        return profileObj;
    }
    async reconcileTabs(profileObj) {
        let utils = tabDefinitionRetriever_1.default.getInstance(this.org);
        if (profileObj.tabVisibilities !== undefined) {
            if (!Array.isArray(profileObj.tabVisibilities)) {
                profileObj.tabVisibilities = [profileObj.tabVisibilities];
            }
            let validArray = [];
            for (let i = 0; i < profileObj.tabVisibilities.length; i++) {
                let cmpObj = profileObj.tabVisibilities[i];
                let exist = await utils.tabExists(cmpObj.tab);
                if (exist) {
                    validArray.push(cmpObj);
                }
            }
            sfpowerkit_1.SFPowerkit.log(`Tab Visibilities reduced from ${profileObj.tabVisibilities.length}  to  ${validArray.length}`, core_1.LoggerLevel.DEBUG);
            profileObj.tabVisibilities = validArray;
        }
        return profileObj;
    }
    async removePermissions(profileObj) {
        sfpowerkit_1.SFPowerkit.log("Reconciling App", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileApp(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Classes", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileClasses(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Fields", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileFields(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Objects", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileObjects(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Pages", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcilePages(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Layouts", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileLayouts(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling Record Types", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileRecordTypes(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  Tabs", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileTabs(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  ExternalDataSources", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileExternalDataSource(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  CustomPermissions", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileCustomPermission(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  CustomMetadata", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileCustomMetadata(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  CustomSettings", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileCustomSettins(profileObj);
        sfpowerkit_1.SFPowerkit.log("Reconciling  Flow", core_1.LoggerLevel.DEBUG);
        profileObj = await this.reconcileFlow(profileObj);
        return profileObj;
    }
}
exports.default = ProfileReconcile;
//# sourceMappingURL=profileReconcile.js.map