"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SFPowerkit = exports.LoggerLevel = void 0;
const core_1 = require("@salesforce/core");
const util_1 = require("util");
const pino_1 = __importDefault(require("pino"));
var LoggerLevel;
(function (LoggerLevel) {
    LoggerLevel[LoggerLevel["TRACE"] = 10] = "TRACE";
    LoggerLevel[LoggerLevel["DEBUG"] = 20] = "DEBUG";
    LoggerLevel[LoggerLevel["INFO"] = 30] = "INFO";
    LoggerLevel[LoggerLevel["WARN"] = 40] = "WARN";
    LoggerLevel[LoggerLevel["ERROR"] = 50] = "ERROR";
    LoggerLevel[LoggerLevel["FATAL"] = 60] = "FATAL";
})(LoggerLevel = exports.LoggerLevel || (exports.LoggerLevel = {}));
class SFPowerkit {
    static setLogLevel(logLevel, isJsonFormatEnabled) {
        logLevel = logLevel.toLowerCase();
        this.isJsonFormatEnabled = isJsonFormatEnabled;
        if (!isJsonFormatEnabled) {
            SFPowerkit.logger = pino_1.default({
                name: "sfpowerkit",
                level: logLevel,
                prettyPrint: {
                    levelFirst: true,
                    colorize: true,
                    translateTime: true,
                    ignore: "pid,hostname" // --ignore
                }
            });
        }
        else {
            //do nothing for now, need to put pino to move to file
        }
        switch (logLevel) {
            case "trace":
                SFPowerkit.logLevel = LoggerLevel.TRACE;
                break;
            case "debug":
                SFPowerkit.logLevel = LoggerLevel.DEBUG;
                break;
            case "info":
                SFPowerkit.logLevel = LoggerLevel.INFO;
                break;
            case "warn":
                SFPowerkit.logLevel = LoggerLevel.WARN;
                break;
            case "error":
                SFPowerkit.logLevel = LoggerLevel.ERROR;
                break;
            case "fatal":
                SFPowerkit.logLevel = LoggerLevel.FATAL;
                break;
        }
    }
    static setProjectDirectories(packagedirectories) {
        SFPowerkit.projectDirectories = packagedirectories;
    }
    static async getProjectDirectories() {
        if (!SFPowerkit.projectDirectories) {
            SFPowerkit.projectDirectories = [];
            const dxProject = await core_1.SfdxProject.resolve();
            const project = await dxProject.retrieveSfdxProjectJson();
            let packages = project.get("packageDirectories") || [];
            packages.forEach(element => {
                SFPowerkit.projectDirectories.push(element.path);
                if (element.default) {
                    SFPowerkit.defaultFolder = element.path;
                }
            });
        }
        return SFPowerkit.projectDirectories;
    }
    static async getDefaultFolder() {
        if (!SFPowerkit.defaultFolder) {
            await SFPowerkit.getProjectDirectories();
        }
        return SFPowerkit.defaultFolder;
    }
    static setDefaultFolder(defaultFolder) {
        SFPowerkit.defaultFolder = defaultFolder;
    }
    static async getConfig() {
        if (!SFPowerkit.pluginConfig) {
            const dxProject = await core_1.SfdxProject.resolve();
            const project = await dxProject.retrieveSfdxProjectJson();
            let plugins = project.get("plugins") || {};
            let sfpowerkitConfig = plugins["sfpowerkit"];
            SFPowerkit.pluginConfig = sfpowerkitConfig || {};
        }
        return SFPowerkit.pluginConfig;
    }
    static setapiversion(apiversion) {
        SFPowerkit.sourceApiVersion = apiversion;
    }
    static async getApiVersion() {
        if (!SFPowerkit.sourceApiVersion) {
            const dxProject = await core_1.SfdxProject.resolve();
            const project = await dxProject.retrieveSfdxProjectJson();
            SFPowerkit.sourceApiVersion = project.get("sourceApiVersion");
        }
        return SFPowerkit.sourceApiVersion;
    }
    /**
     * Print log only if the log level for this commamnd matches the log level for the message
     * @param message Message to print
     * @param messageLoglevel Log level for the message
     */
    static log(message, logLevel) {
        if (util_1.isNullOrUndefined(this.logger))
            return;
        if (this.isJsonFormatEnabled)
            return;
        switch (logLevel) {
            case LoggerLevel.TRACE:
                this.logger.trace(message);
                break;
            case LoggerLevel.DEBUG:
                this.logger.debug(message);
                break;
            case LoggerLevel.INFO:
                this.logger.info(message);
                break;
            case LoggerLevel.WARN:
                this.logger.warn(message);
                break;
            case LoggerLevel.ERROR:
                this.logger.error(message);
                break;
            case LoggerLevel.FATAL:
                this.logger.fatal(message);
                break;
        }
    }
    static setUx(ux) {
        this.ux = ux;
    }
    static setStatus(status) {
        this.ux.setSpinnerStatus(status);
    }
}
exports.SFPowerkit = SFPowerkit;
//# sourceMappingURL=sfpowerkit.js.map